# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

import swiftclient.client
import testscenarios
import testtools
from testtools import matchers
import time

from heatclient.common import deployment_utils
from heatclient import exc
from heatclient.v1 import software_configs


load_tests = testscenarios.load_tests_apply_scenarios


def mock_sc(group=None, config=None, options=None,
            inputs=None, outputs=None):
    return software_configs.SoftwareConfig(None, {
        'group': group,
        'config': config,
        'options': options or {},
        'inputs': inputs or [],
        'outputs': outputs or [],
    }, True)


class DerivedConfigTest(testtools.TestCase):

    scenarios = [
        ('defaults', dict(
            action='UPDATE',
            source=mock_sc(),
            name='s1',
            input_values=None,
            server_id='1234',
            signal_transport='NO_SIGNAL',
            signal_id=None,
            result={
                'config': '',
                'group': 'Heat::Ungrouped',
                'inputs': [{
                    'description': 'ID of the server being deployed to',
                    'name': 'deploy_server_id',
                    'type': 'String',
                    'value': '1234'
                }, {
                    'description': 'Name of the current action '
                    'being deployed',
                    'name': 'deploy_action',
                    'type': 'String',
                    'value': 'UPDATE'
                }, {
                    'description': 'How the server should signal to '
                    'heat with the deployment output values.',
                    'name': 'deploy_signal_transport',
                    'type': 'String',
                    'value': 'NO_SIGNAL'}],
                'name': 's1',
                'options': {},
                'outputs': []})),
        ('defaults_empty', dict(
            action='UPDATE',
            source={},
            name='s1',
            input_values=None,
            server_id='1234',
            signal_transport='NO_SIGNAL',
            signal_id=None,
            result={
                'config': '',
                'group': 'Heat::Ungrouped',
                'inputs': [{
                    'description': 'ID of the server being deployed to',
                    'name': 'deploy_server_id',
                    'type': 'String',
                    'value': '1234'
                }, {
                    'description': 'Name of the current action '
                    'being deployed',
                    'name': 'deploy_action',
                    'type': 'String',
                    'value': 'UPDATE'
                }, {
                    'description': 'How the server should signal to '
                    'heat with the deployment output values.',
                    'name': 'deploy_signal_transport',
                    'type': 'String',
                    'value': 'NO_SIGNAL'}],
                'name': 's1',
                'options': {},
                'outputs': []})),

        ('config_values', dict(
            action='UPDATE',
            source=mock_sc(
                group='puppet',
                config='do the foo',
                inputs=[
                    {'name': 'one', 'default': '1'},
                    {'name': 'two'}],
                options={'option1': 'value'},
                outputs=[
                    {'name': 'output1'},
                    {'name': 'output2'}],
            ),
            name='s2',
            input_values={'one': 'foo', 'two': 'bar', 'three': 'baz'},
            server_id='1234',
            signal_transport='NO_SIGNAL',
            signal_id=None,
            result={
                'config': 'do the foo',
                'group': 'puppet',
                'inputs': [{
                    'name': 'one',
                    'default': '1',
                    'value': 'foo'
                }, {
                    'name': 'two',
                    'value': 'bar'
                }, {
                    'name': 'three',
                    'type': 'String',
                    'value': 'baz'
                }, {
                    'description': 'ID of the server being deployed to',
                    'name': 'deploy_server_id',
                    'type': 'String',
                    'value': '1234'
                }, {
                    'description': 'Name of the current action '
                    'being deployed',
                    'name': 'deploy_action',
                    'type': 'String',
                    'value': 'UPDATE'
                }, {
                    'description': 'How the server should signal to '
                    'heat with the deployment output values.',
                    'name': 'deploy_signal_transport',
                    'type': 'String',
                    'value': 'NO_SIGNAL'
                }],
                'name': 's2',
                'options': {'option1': 'value'},
                'outputs': [
                    {'name': 'output1'},
                    {'name': 'output2'}]})),
        ('temp_url', dict(
            action='UPDATE',
            source=mock_sc(),
            name='s1',
            input_values=None,
            server_id='1234',
            signal_transport='TEMP_URL_SIGNAL',
            signal_id='http://192.0.2.1:8080/foo',
            result={
                'config': '',
                'group': 'Heat::Ungrouped',
                'inputs': [{
                    'description': 'ID of the server being deployed to',
                    'name': 'deploy_server_id',
                    'type': 'String',
                    'value': '1234'
                }, {
                    'description': 'Name of the current action '
                    'being deployed',
                    'name': 'deploy_action',
                    'type': 'String',
                    'value': 'UPDATE'
                }, {
                    'description': 'How the server should signal to '
                    'heat with the deployment output values.',
                    'name': 'deploy_signal_transport',
                    'type': 'String',
                    'value': 'TEMP_URL_SIGNAL'
                }, {
                    'description': 'ID of signal to use for signaling '
                    'output values',
                    'name': 'deploy_signal_id',
                    'type': 'String',
                    'value': 'http://192.0.2.1:8080/foo'
                }, {
                    'description': 'HTTP verb to use for signaling '
                    'output values',
                    'name': 'deploy_signal_verb',
                    'type': 'String',
                    'value': 'PUT'}],
                'name': 's1',
                'options': {},
                'outputs': []})),
        ('unsupported', dict(
            action='UPDATE',
            source=mock_sc(),
            name='s1',
            input_values=None,
            server_id='1234',
            signal_transport='ASDF',
            signal_id=None,
            result_error=exc.CommandError,
            result_error_msg='Unsupported signal transport ASDF',
            result=None)),
    ]

    def test_build_derived_config_params(self):
        try:
            self.assertEqual(
                self.result,
                deployment_utils.build_derived_config_params(
                    action=self.action,
                    source=self.source,
                    name=self.name,
                    input_values=self.input_values,
                    server_id=self.server_id,
                    signal_transport=self.signal_transport,
                    signal_id=self.signal_id))
        except Exception as e:
            if not self.result_error:
                raise e
            self.assertIsInstance(e, self.result_error)
            self.assertEqual(self.result_error_msg, str(e))


class TempURLSignalTest(testtools.TestCase):

    @mock.patch.object(swiftclient.client, 'Connection')
    def test_create_swift_client(self, sc_conn):
        auth = mock.MagicMock()
        auth.get_token.return_value = '1234'
        auth.get_endpoint.return_value = 'http://192.0.2.1:8080'

        session = mock.MagicMock()

        args = mock.MagicMock()
        args.os_region_name = 'Region1'
        args.os_project_name = 'project'
        args.os_username = 'user'
        args.os_cacert = None
        args.insecure = True

        sc_conn.return_value = mock.MagicMock()

        sc = deployment_utils.create_swift_client(auth, session, args)

        self.assertEqual(sc_conn.return_value, sc)

        self.assertEqual(
            mock.call(session),
            auth.get_token.call_args)

        self.assertEqual(
            mock.call(
                session,
                service_type='object-store',
                region_name='Region1'),
            auth.get_endpoint.call_args)

        self.assertEqual(
            mock.call(
                cacert=None,
                insecure=True,
                key=None,
                tenant_name='project',
                preauthtoken='1234',
                authurl=None,
                user='user',
                preauthurl='http://192.0.2.1:8080',
                auth_version='2.0'),
            sc_conn.call_args)

    def test_create_temp_url(self):
        swift_client = mock.MagicMock()
        swift_client.url = ("http://fake-host.com:8080/v1/AUTH_demo")
        swift_client.head_account = mock.Mock(return_value={
            'x-account-meta-temp-url-key': '123456'})
        swift_client.post_account = mock.Mock()

        uuid_pattern = ('[a-f0-9]{8}-[a-f0-9]{4}-4[a-f0-9]{3}-[89aAbB]'
                        '[a-f0-9]{3}-[a-f0-9]{12}')
        url = deployment_utils.create_temp_url(swift_client, 'bar', 60)
        self.assertFalse(swift_client.post_account.called)
        regexp = (r"http://fake-host.com:8080/v1/AUTH_demo/bar-%s"
                  r"/%s\?temp_url_sig=[0-9a-f]{40,64}&"
                  r"temp_url_expires=[0-9]{10}" % (uuid_pattern, uuid_pattern))
        self.assertThat(url, matchers.MatchesRegex(regexp))

        timeout = int(url.split('=')[-1])
        self.assertTrue(timeout < time.time() + 2*365*24*60*60)

    def test_get_temp_url_no_account_key(self):
        swift_client = mock.MagicMock()
        swift_client.url = ("http://fake-host.com:8080/v1/AUTH_demo")
        head_account = {}

        def post_account(data):
            head_account.update(data)

        swift_client.head_account = mock.Mock(return_value=head_account)
        swift_client.post_account = post_account

        self.assertNotIn('x-account-meta-temp-url-key', head_account)
        deployment_utils.create_temp_url(swift_client, 'bar', 60, 'foo')
        self.assertIn('x-account-meta-temp-url-key', head_account)

    def test_build_signal_id_no_signal(self):
        hc = mock.MagicMock()
        args = mock.MagicMock()
        args.signal_transport = 'NO_SIGNAL'
        self.assertIsNone(deployment_utils.build_signal_id(hc, args))

    def test_build_signal_id_no_client_auth(self):
        hc = mock.MagicMock()
        args = mock.MagicMock()
        args.os_no_client_auth = True
        args.signal_transport = 'TEMP_URL_SIGNAL'
        e = self.assertRaises(exc.CommandError,
                              deployment_utils.build_signal_id, hc, args)
        self.assertEqual((
            'Cannot use --os-no-client-auth, auth required to create '
            'a Swift TempURL.'),
            str(e))

    @mock.patch.object(deployment_utils, 'create_temp_url')
    @mock.patch.object(deployment_utils, 'create_swift_client')
    def test_build_signal_id(self, csc, ctu):
        hc = mock.MagicMock()
        args = mock.MagicMock()
        args.name = 'foo'
        args.timeout = 60
        args.os_no_client_auth = False
        args.signal_transport = 'TEMP_URL_SIGNAL'
        csc.return_value = mock.MagicMock()
        temp_url = (
            'http://fake-host.com:8080/v1/AUTH_demo/foo/'
            'a81a74d5-c395-4269-9670-ddd0824fd696'
            '?temp_url_sig=6a68371d602c7a14aaaa9e3b3a63b8b85bd9a503'
            '&temp_url_expires=1425270977')
        ctu.return_value = temp_url

        self.assertEqual(
            temp_url, deployment_utils.build_signal_id(hc, args))
        self.assertEqual(
            mock.call(hc.http_client.auth, hc.http_client.session, args),
            csc.call_args)
        self.assertEqual(
            mock.call(csc.return_value, 'foo', 60),
            ctu.call_args)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from cliff import columns

from heatclient.common import utils as heat_utils


class LinkColumn(columns.FormattableColumn):
    def human_readable(self):
        return heat_utils.link_formatter(self._value)


class JsonColumn(columns.FormattableColumn):
    def human_readable(self):
        return heat_utils.json_formatter(self._value)


class YamlColumn(columns.FormattableColumn):
    def human_readable(self):
        return heat_utils.yaml_formatter(self._value)


class TextWrapColumn(columns.FormattableColumn):
    def human_readable(self):
        return heat_utils.text_wrap_formatter(self._value)


class NewlineListColumn(columns.FormattableColumn):
    def human_readable(self):
        return heat_utils.newline_list_formatter(self._value)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from heatclient.v1 import template_versions


class TemplateVersionManagerTest(testtools.TestCase):

    def setUp(self):
        super(TemplateVersionManagerTest, self).setUp()

    def test_list_versions(self):
        expect = ('GET', '/template_versions')

        class FakeResponse(object):
            def json(self):
                return {'template_versions': [{'version': '2013-05-23',
                                               'type': 'hot'}]}

        class FakeClient(object):
            def get(self, *args, **kwargs):
                assert ('GET', args[0]) == expect
                return FakeResponse()

        manager = template_versions.TemplateVersionManager(FakeClient())
        versions = manager.list()
        self.assertEqual('2013-05-23', getattr(versions[0], 'version'))
        self.assertEqual('hot', getattr(versions[0], 'type'))

    def test_get(self):
        expect = ('GET', '/template_versions/heat_template_version.2015-04-30'
                         '/functions')

        class FakeResponse(object):
            def json(self):
                return {'template_functions': [{'function': 'get_attr'}]}

        class FakeClient(object):
            def get(self, *args, **kwargs):
                assert ('GET', args[0]) == expect
                return FakeResponse()

        manager = template_versions.TemplateVersionManager(FakeClient())
        functions = manager.get('heat_template_version.2015-04-30')
        self.assertEqual('get_attr', getattr(functions[0], 'function'))



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
import hashlib
import logging
import os
import socket

from keystoneauth1 import adapter
from oslo_serialization import jsonutils
from oslo_utils import encodeutils
from oslo_utils import importutils
import requests
from urllib import parse

from heatclient._i18n import _
from heatclient.common import utils
from heatclient import exc

LOG = logging.getLogger(__name__)
USER_AGENT = 'python-heatclient'
CHUNKSIZE = 1024 * 64  # 64kB
SENSITIVE_HEADERS = ('X-Auth-Token',)
osprofiler_web = importutils.try_import("osprofiler.web")


def authenticated_fetcher(hc):
    """A wrapper around the heat client object to fetch a template."""

    def _do(*args, **kwargs):
        if isinstance(hc.http_client, SessionClient):
            method, url = args
            return hc.http_client.request(url, method, **kwargs).content
        else:
            return hc.http_client.raw_request(*args, **kwargs).content

    return _do


def get_system_ca_file():
    """Return path to system default CA file."""
    # Standard CA file locations for Debian/Ubuntu, RedHat/Fedora,
    # Suse, FreeBSD/OpenBSD, MacOSX, and the bundled ca
    ca_path = ['/etc/ssl/certs/ca-certificates.crt',
               '/etc/pki/tls/certs/ca-bundle.crt',
               '/etc/ssl/ca-bundle.pem',
               '/etc/ssl/cert.pem',
               '/System/Library/OpenSSL/certs/cacert.pem',
               requests.certs.where()]
    for ca in ca_path:
        LOG.debug("Looking for ca file %s", ca)
        if os.path.exists(ca):
            LOG.debug("Using ca file %s", ca)
            return ca
    LOG.warning("System ca file could not be found.")


class HTTPClient(object):

    def __init__(self, endpoint, **kwargs):
        self.endpoint = endpoint
        self.auth_url = kwargs.get('auth_url')
        self.auth_token = kwargs.get('token')
        self.username = kwargs.get('username')
        self.password = kwargs.get('password')
        self.region_name = kwargs.get('region_name')
        self.include_pass = kwargs.get('include_pass')
        self.endpoint_url = endpoint

        self.cert_file = kwargs.get('cert_file')
        self.key_file = kwargs.get('key_file')
        self.timeout = kwargs.get('timeout')

        self.ssl_connection_params = {
            'ca_file': kwargs.get('ca_file'),
            'cert_file': kwargs.get('cert_file'),
            'key_file': kwargs.get('key_file'),
            'insecure': kwargs.get('insecure'),
        }

        self.verify_cert = None
        if parse.urlparse(endpoint).scheme == "https":
            if kwargs.get('insecure'):
                self.verify_cert = False
            else:
                self.verify_cert = kwargs.get('ca_file', get_system_ca_file())

        # FIXME(shardy): We need this for compatibility with the oslo apiclient
        # we should move to inheriting this class from the oslo HTTPClient
        self.last_request_id = None

    def safe_header(self, name, value):
        if name in SENSITIVE_HEADERS:
            # because in python3 byte string handling is ... ug
            v = value.encode('utf-8')
            h = hashlib.sha1(v)
            d = h.hexdigest()
            return encodeutils.safe_decode(name), "{SHA1}%s" % d
        else:
            return (encodeutils.safe_decode(name),
                    encodeutils.safe_decode(value))

    def log_curl_request(self, method, url, kwargs):
        curl = ['curl -g -i -X %s' % method]

        for (key, value) in kwargs['headers'].items():
            header = '-H \'%s: %s\'' % self.safe_header(key, value)
            curl.append(header)

        conn_params_fmt = [
            ('key_file', '--key %s'),
            ('cert_file', '--cert %s'),
            ('ca_file', '--cacert %s'),
        ]
        for (key, fmt) in conn_params_fmt:
            value = self.ssl_connection_params.get(key)
            if value:
                curl.append(fmt % value)

        if self.ssl_connection_params.get('insecure'):
            curl.append('-k')

        if 'data' in kwargs:
            curl.append('-d \'%s\'' % kwargs['data'])

        if not parse.urlparse(url).netloc:
            url = self.endpoint + url

        curl.append(url)
        LOG.debug(' '.join(curl))

    @staticmethod
    def log_http_response(resp):
        status = (resp.raw.version / 10.0, resp.status_code, resp.reason)
        dump = ['\nHTTP/%.1f %s %s' % status]
        dump.extend(['%s: %s' % (k, v) for k, v in resp.headers.items()])
        dump.append('')
        if resp.content:
            content = resp.content
            if isinstance(content, bytes):
                content = content.decode()
            dump.extend([content, ''])
        LOG.debug('\n'.join(dump))

    def _http_request(self, url, method, **kwargs):
        """Send an http request with the specified characteristics.

        Wrapper around requests.request to handle tasks such as
        setting headers and error handling.
        """
        # Copy the kwargs so we can reuse the original in case of redirects
        kwargs['headers'] = copy.deepcopy(kwargs.get('headers', {}))
        kwargs['headers'].setdefault('User-Agent', USER_AGENT)
        if self.auth_token:
            kwargs['headers'].setdefault('X-Auth-Token', self.auth_token)
        else:
            kwargs['headers'].update(self.credentials_headers())
        if self.auth_url:
            kwargs['headers'].setdefault('X-Auth-Url', self.auth_url)
        if self.region_name:
            kwargs['headers'].setdefault('X-Region-Name', self.region_name)
        if self.include_pass and 'X-Auth-Key' not in kwargs['headers']:
            kwargs['headers'].update(self.credentials_headers())
        if osprofiler_web:
            kwargs['headers'].update(osprofiler_web.get_trace_id_headers())

        self.log_curl_request(method, url, kwargs)

        if self.cert_file and self.key_file:
            kwargs['cert'] = (self.cert_file, self.key_file)

        if self.verify_cert is not None:
            kwargs['verify'] = self.verify_cert

        if self.timeout is not None:
            kwargs['timeout'] = float(self.timeout)

        # Allow caller to specify not to follow redirects, in which case we
        # just return the redirect response.  Useful for using stacks:lookup.
        redirect = kwargs.pop('redirect', True)

        # Since requests does not follow the RFC when doing redirection to sent
        # back the same method on a redirect we are simply bypassing it.  For
        # example if we do a DELETE/POST/PUT on a URL and we get a 302 RFC says
        # that we should follow that URL with the same method as before,
        # requests doesn't follow that and send a GET instead for the method.
        # Hopefully this could be fixed as they say in a comment in a future
        # point version i.e.: 3.x
        # See issue: https://github.com/kennethreitz/requests/issues/1704
        allow_redirects = False

        # Use fully qualified URL from response header for redirects
        if not parse.urlparse(url).netloc:
            url = self.endpoint_url + url

        try:
            resp = requests.request(
                method,
                url,
                allow_redirects=allow_redirects,
                **kwargs)
        except socket.gaierror as e:
            message = (_("Error finding address for %(url)s: %(e)s") %
                       {'url': self.endpoint_url + url, 'e': e})
            raise exc.InvalidEndpoint(message=message)
        except (socket.error, socket.timeout) as e:
            endpoint = self.endpoint
            message = (_("Error communicating with %(endpoint)s %(e)s") %
                       {'endpoint': endpoint, 'e': e})
            raise exc.CommunicationError(message=message)

        self.log_http_response(resp)
        txt_content = encodeutils.safe_decode(resp.content, 'utf-8')
        if not ('X-Auth-Key' in kwargs['headers']) and (
                resp.status_code == 401 or
                (resp.status_code == 500 and "(HTTP 401)" in txt_content)):
            raise exc.HTTPUnauthorized(_("Authentication failed: %s")
                                       % resp.content)
        elif 400 <= resp.status_code < 600:
            raise exc.from_response(resp)
        elif resp.status_code in (301, 302, 305):
            # Redirected. Reissue the request to the new location,
            # unless caller specified redirect=False
            if redirect:
                location = resp.headers.get('location')
                if not location:
                    message = _("Location not returned with redirect")
                    raise exc.InvalidEndpoint(message=message)
                resp = self._http_request(location, method, **kwargs)
        elif resp.status_code == 300:
            raise exc.from_response(resp)

        return resp

    def credentials_headers(self):
        creds = {}
        # NOTE(dhu): (shardy) When deferred_auth_method=password, Heat
        # encrypts and stores username/password.  For Keystone v3, the
        # intent is to use trusts since SHARDY is working towards
        # deferred_auth_method=trusts as the default.
        # TODO(dhu): Make Keystone v3 work in Heat standalone mode.  Maye
        # require X-Auth-User-Domain.
        if self.username:
            creds['X-Auth-User'] = self.username
        if self.password:
            creds['X-Auth-Key'] = self.password
        return creds

    def json_request(self, method, url, **kwargs):
        kwargs.setdefault('headers', {})
        kwargs['headers'].setdefault('Content-Type', 'application/json')
        kwargs['headers'].setdefault('Accept', 'application/json')

        if 'data' in kwargs:
            kwargs['data'] = jsonutils.dumps(kwargs['data'])

        resp = self._http_request(url, method, **kwargs)
        body = utils.get_response_body(resp)
        return resp, body

    def raw_request(self, method, url, **kwargs):
        kwargs.setdefault('headers', {})
        kwargs['headers'].setdefault('Content-Type',
                                     'application/octet-stream')
        return self._http_request(url, method, **kwargs)

    def client_request(self, method, url, **kwargs):
        resp, body = self.json_request(method, url, **kwargs)
        return resp

    def head(self, url, **kwargs):
        return self.client_request("HEAD", url, **kwargs)

    def get(self, url, **kwargs):
        return self.client_request("GET", url, **kwargs)

    def post(self, url, **kwargs):
        return self.client_request("POST", url, **kwargs)

    def put(self, url, **kwargs):
        return self.client_request("PUT", url, **kwargs)

    def delete(self, url, **kwargs):
        return self.raw_request("DELETE", url, **kwargs)

    def patch(self, url, **kwargs):
        return self.client_request("PATCH", url, **kwargs)


class SessionClient(adapter.LegacyJsonAdapter):
    """HTTP client based on Keystone client session."""

    def request(self, url, method, **kwargs):
        redirect = kwargs.get('redirect')
        kwargs.setdefault('user_agent', USER_AGENT)

        headers = kwargs.setdefault('headers', {})
        headers.setdefault('Content-Type', 'application/json')

        if 'data' in kwargs:
            kwargs['data'] = jsonutils.dumps(kwargs['data'])

        resp, body = super(SessionClient, self).request(
            url, method,
            raise_exc=False,
            **kwargs)

        if 400 <= resp.status_code < 600:
            raise exc.from_response(resp)
        elif resp.status_code in (301, 302, 305):
            if redirect:
                location = resp.headers.get('location')
                path = self.strip_endpoint(location)
                resp = self.request(path, method, **kwargs)
        elif resp.status_code == 300:
            raise exc.from_response(resp)

        return resp

    def credentials_headers(self):
        return {}

    def strip_endpoint(self, location):
        if location is None:
            message = _("Location not returned with 302")
            raise exc.InvalidEndpoint(message=message)
        if (self.endpoint_override is not None and
                location.lower().startswith(self.endpoint_override.lower())):
            return location[len(self.endpoint_override):]
        else:
            return location


def _construct_http_client(endpoint=None, username=None, password=None,
                           include_pass=None, endpoint_type=None,
                           auth_url=None, **kwargs):
    session = kwargs.pop('session', None)
    auth = kwargs.pop('auth', None)

    if session:
        if 'endpoint_override' not in kwargs and endpoint:
            kwargs['endpoint_override'] = endpoint

        if 'service_type' not in kwargs:
            kwargs['service_type'] = 'orchestration'

        if 'interface' not in kwargs and endpoint_type:
            kwargs['interface'] = endpoint_type

        if 'region_name' in kwargs:
            kwargs['additional_headers'] = {
                'X-Region-Name': kwargs['region_name']}

        return SessionClient(session, auth=auth, **kwargs)
    else:
        return HTTPClient(endpoint=endpoint, username=username,
                          password=password, include_pass=include_pass,
                          endpoint_type=endpoint_type, auth_url=auth_url,
                          **kwargs)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from urllib import parse

from heatclient.common import base
from heatclient.common import utils


class SoftwareDeployment(base.Resource):
    def __repr__(self):
        return "<SoftwareDeployment %s>" % self._info

    def update(self, **fields):
        self.manager.update(deployment_id=self.id, **fields)

    def delete(self):
        return self.manager.delete(deployment_id=self.id)


class SoftwareDeploymentManager(base.BaseManager):
    resource_class = SoftwareDeployment

    def list(self, **kwargs):
        """Get a list of software deployments.

        :rtype: list of :class:`SoftwareDeployment`
        """
        url = '/software_deployments?%s' % parse.urlencode(kwargs)
        return self._list(url, "software_deployments")

    def metadata(self, server_id):
        """Get a collection of software deployment metadata for given server.

        :rtype: list of :class:`SoftwareDeployment`
        """
        url = '/software_deployments/metadata/%s' % parse.quote(
            server_id)
        resp = self.client.get(url)
        body = utils.get_response_body(resp)
        return body.get('metadata')

    def get(self, deployment_id):
        """Get the details for a specific software deployment.

        :param deployment_id: ID of the software deployment
        """
        resp = self.client.get('/software_deployments/%s' % deployment_id)
        body = utils.get_response_body(resp)

        return SoftwareDeployment(self, body.get('software_deployment'))

    def create(self, **kwargs):
        """Create a software deployment."""
        resp = self.client.post('/software_deployments', data=kwargs)
        body = utils.get_response_body(resp)
        return SoftwareDeployment(self, body.get('software_deployment'))

    def update(self, deployment_id, **kwargs):
        """Update a software deployment."""
        resp = self.client.put('/software_deployments/%s' %
                               deployment_id, data=kwargs)
        body = utils.get_response_body(resp)
        return SoftwareDeployment(self, body.get('software_deployment'))

    def delete(self, deployment_id):
        """Delete a software deployment."""
        self._delete("/software_deployments/%s" % deployment_id)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from heatclient.common import base
from heatclient.v1 import stacks

DEFAULT_PAGE_SIZE = 20


class Action(base.Resource):
    def __repr__(self):
        return "<Action %s>" % self._info

    def update(self, **fields):
        self.manager.update(self, **fields)

    def delete(self):
        return self.manager.delete(self)

    def data(self, **kwargs):
        return self.manager.data(self, **kwargs)


class ActionManager(stacks.StackChildManager):
    resource_class = Action

    def suspend(self, stack_id):
        """Suspend a stack."""
        body = {'suspend': None}
        self.client.post('/stacks/%s/actions' % stack_id, data=body)

    def resume(self, stack_id):
        """Resume a stack."""
        body = {'resume': None}
        self.client.post('/stacks/%s/actions' % stack_id, data=body)

    def cancel_update(self, stack_id):
        """Cancel running update of a stack."""
        body = {'cancel_update': None}
        self.client.post('/stacks/%s/actions' % stack_id, data=body)

    def cancel_without_rollback(self, stack_id):
        """Cancel running update of a stack."""
        body = {'cancel_without_rollback': None}
        self.client.post('/stacks/%s/actions' % stack_id, data=body)

    def check(self, stack_id):
        """Check a stack."""
        body = {'check': None}
        self.client.post('/stacks/%s/actions' % stack_id, data=body)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

"""Orchestration v1 Stack action implementations"""

import logging

from osc_lib.command import command
from osc_lib import exceptions as exc
from osc_lib.i18n import _
from osc_lib import utils
from oslo_serialization import jsonutils
from urllib import request

from heatclient.common import format_utils
from heatclient.common import utils as heat_utils
from heatclient import exc as heat_exc


class ResourceShow(command.ShowOne):
    """Display stack resource."""

    log = logging.getLogger(__name__ + '.ResourceShowStack')

    def get_parser(self, prog_name):
        parser = super(ResourceShow, self).get_parser(prog_name)
        parser.add_argument(
            'stack',
            metavar='<stack>',
            help=_('Name or ID of stack to query')
        )
        parser.add_argument(
            'resource',
            metavar='<resource>',
            help=_('Name of resource')
        )
        parser.add_argument(
            '--with-attr',
            metavar='<attribute>',
            action='append',
            help=_('Attribute to show, can be specified multiple times')
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)', parsed_args)

        client = self.app.client_manager.orchestration

        try:
            resource = client.resources.get(parsed_args.stack,
                                            parsed_args.resource,
                                            with_attr=parsed_args.with_attr)
        except heat_exc.HTTPNotFound:
            msg = (_('Stack or resource not found: %(stack)s %(resource)s') %
                   {'stack': parsed_args.stack,
                    'resource': parsed_args.resource})
            raise exc.CommandError(msg)

        return self.dict2columns(resource.to_dict())


class ResourceList(command.Lister):
    """List stack resources."""

    log = logging.getLogger(__name__ + '.ResourceListStack')

    @property
    def formatter_namespace(self):
        return 'heatclient.resource.formatter.list'

    def get_parser(self, prog_name):
        parser = super(ResourceList, self).get_parser(prog_name)
        parser.add_argument(
            'stack',
            metavar='<stack>',
            help=_('Name or ID of stack to query')
        )
        parser.add_argument(
            '--long',
            action='store_true',
            help=_('Enable detailed information presented for each resource '
                   'in resource list')
        )
        parser.add_argument(
            '-n', '--nested-depth',
            metavar='<nested-depth>',
            type=int,
            help=_('Depth of nested stacks from which to display resources')
        )

        parser.add_argument(
            '--filter',
            metavar='<key=value>',
            action='append',
            help=_('Filter parameters to apply on returned resources based on '
                   'their name, status, type, action, id and '
                   'physical_resource_id')
        )

        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)', parsed_args)

        client = self.app.client_manager.orchestration

        fields = {
            'nested_depth': parsed_args.nested_depth,
            'with_detail': parsed_args.long,
            'filters': heat_utils.format_parameters(parsed_args.filter),
        }

        try:
            resources = client.resources.list(parsed_args.stack, **fields)
        except heat_exc.HTTPNotFound:
            msg = _('Stack not found: %s') % parsed_args.stack
            raise exc.CommandError(msg)

        if parsed_args.formatter == 'dot':
            return [], resources

        columns = ['physical_resource_id', 'resource_type', 'resource_status',
                   'updated_time']

        if len(resources) >= 1 and not hasattr(resources[0], 'resource_name'):
            columns.insert(0, 'logical_resource_id')
        else:
            columns.insert(0, 'resource_name')

        if parsed_args.nested_depth or parsed_args.long:
            columns.append('stack_name')

        return (
            columns,
            (utils.get_item_properties(r, columns) for r in resources)
        )


class ResourceMetadata(format_utils.JsonFormat):
    """Show resource metadata"""

    log = logging.getLogger(__name__ + ".ResourceMetadata")

    def get_parser(self, prog_name):
        parser = super(ResourceMetadata, self).get_parser(prog_name)
        parser.add_argument(
            'stack',
            metavar='<stack>',
            help=_('Stack to display (name or ID)'),
        )
        parser.add_argument(
            'resource',
            metavar='<resource>',
            help=_('Name of the resource to show the metadata for'))
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)

        heat_client = self.app.client_manager.orchestration
        return _resource_metadata(heat_client, parsed_args)


def _resource_metadata(heat_client, args):
    fields = {'stack_id': args.stack,
              'resource_name': args.resource}
    try:
        metadata = heat_client.resources.metadata(**fields)
    except heat_exc.HTTPNotFound:
        raise exc.CommandError(_('Stack %(stack)s or resource %(resource)s '
                                 'not found.') %
                               {'stack': args.stack,
                                'resource': args.resource})

    data = list(metadata.values())
    columns = list(metadata.keys())
    return columns, data


class ResourceSignal(command.Command):
    """Signal a resource with optional data."""

    log = logging.getLogger(__name__ + ".ResourceSignal")

    def get_parser(self, prog_name):
        parser = super(ResourceSignal, self).get_parser(prog_name)
        parser.add_argument(
            'stack',
            metavar='<stack>',
            help=_('Name or ID of stack the resource belongs to'),
        )
        parser.add_argument(
            'resource',
            metavar='<resource>',
            help=_('Name of the resoure to signal'),
        )
        parser.add_argument(
            '--data',
            metavar='<data>',
            help=_('JSON Data to send to the signal handler')
        )
        parser.add_argument(
            '--data-file',
            metavar='<data-file>',
            help=_('File containing JSON data to send to the signal handler')
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)

        heat_client = self.app.client_manager.orchestration
        return _resource_signal(heat_client, parsed_args)


def _resource_signal(heat_client, args):
    fields = {'stack_id': args.stack,
              'resource_name': args.resource}
    data = args.data
    data_file = args.data_file
    if data and data_file:
        raise exc.CommandError(_('Should only specify one of data or '
                                 'data-file'))

    if data_file:
        data_url = heat_utils.normalise_file_path_to_url(data_file)
        data = request.urlopen(data_url).read()

    if data:
        try:
            data = jsonutils.loads(data)
        except ValueError as ex:
            raise exc.CommandError(_('Data should be in JSON format: %s') % ex)
        if not isinstance(data, dict):
            raise exc.CommandError(_('Data should be a JSON dict'))

        fields['data'] = data
    try:
        heat_client.resources.signal(**fields)
    except heat_exc.HTTPNotFound:
        raise exc.CommandError(_('Stack %(stack)s or resource %(resource)s '
                                 'not found.') %
                               {'stack': args.stack,
                                'resource': args.resource})


class ResourceMarkUnhealthy(command.Command):
    """Set resource's health."""

    log = logging.getLogger(__name__ + ".ResourceMarkUnhealthy")

    def get_parser(self, prog_name):
        parser = super(ResourceMarkUnhealthy, self).get_parser(prog_name)
        parser.add_argument(
            'stack',
            metavar='<stack>',
            help=_('Name or ID of stack the resource belongs to')
        )
        parser.add_argument(
            'resource',
            metavar='<resource>',
            help=_('Name of the resource')
        )
        parser.add_argument(
            'reason',
            default="",
            nargs='?',
            help=_('Reason for state change')
        )
        parser.add_argument(
            '--reset',
            default=False,
            action="store_true",
            help=_('Set the resource as healthy')
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)

        heat_client = self.app.client_manager.orchestration
        fields = {'stack_id': parsed_args.stack,
                  'resource_name': parsed_args.resource,
                  'mark_unhealthy': not parsed_args.reset,
                  'resource_status_reason': parsed_args.reason}
        try:
            heat_client.resources.mark_unhealthy(**fields)
        except heat_exc.HTTPNotFound:
            raise exc.CommandError(_('Stack or resource not found: '
                                     '%(id)s %(resource)s') %
                                   {'id': parsed_args.stack,
                                    'resource': parsed_args.resource})



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
#   Copyright 2015 IBM Corp.

from unittest import mock

from heatclient import exc
from heatclient.osc.v1 import template
from heatclient.tests.unit.osc.v1 import fakes
from heatclient.v1 import template_versions


class TestTemplate(fakes.TestOrchestrationv1):
    def setUp(self):
        super(TestTemplate, self).setUp()
        self.mock_client = self.app.client_manager.orchestration
        self.template_versions = self.mock_client.template_versions


class TestTemplateVersionList(TestTemplate):

    def _stub_versions_list(self, ret_data):
        tv1 = template_versions.TemplateVersion(None, ret_data[0])
        tv2 = template_versions.TemplateVersion(None, ret_data[1])
        self.template_versions.list.return_value = [tv1, tv2]
        self.cmd = template.VersionList(self.app, None)

    def test_version_list(self):
        ret_data = [
            {'version': 'HOT123', 'type': 'hot'},
            {'version': 'CFN456', 'type': 'cfn'}]
        self._stub_versions_list(ret_data)
        parsed_args = self.check_parser(self.cmd, [], [])

        columns, data = self.cmd.take_action(parsed_args)

        self.assertEqual(['Version', 'Type'], columns)
        self.assertEqual([('HOT123', 'hot'), ('CFN456', 'cfn')], list(data))

    def test_version_list_with_aliases(self):
        ret_data = [
            {'version': 'HOT123', 'type': 'hot', 'aliases': ['releasex']},
            {'version': 'CFN456', 'type': 'cfn', 'aliases': ['releasey']}]
        self._stub_versions_list(ret_data)
        parsed_args = self.check_parser(self.cmd, [], [])

        columns, data = self.cmd.take_action(parsed_args)

        self.assertEqual(['Version', 'Type', 'Aliases'], columns)
        self.assertEqual([
            ('HOT123', 'hot', template.ListColumn(['releasex'])),
            ('CFN456', 'cfn', template.ListColumn(['releasey']))
        ], list(data))


class TestTemplateFunctionList(TestTemplate):

    defaults = [
        {'functions': 'func1', 'description': 'Function 1'},
        {'functions': 'func2', 'description': 'Function 2'},
        {'functions': 'condition func', 'description': 'Condition Function'}
    ]

    def setUp(self):
        super(TestTemplateFunctionList, self).setUp()
        self.tv1 = template_versions.TemplateVersion(None, self.defaults[0])
        self.tv2 = template_versions.TemplateVersion(None, self.defaults[1])
        self.tv_with_cf = template_versions.TemplateVersion(
            None, self.defaults[2])

        self.cmd = template.FunctionList(self.app, None)

    def test_function_list(self):
        arglist = ['version1']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.template_versions.get.return_value = [self.tv1, self.tv2]
        columns, data = self.cmd.take_action(parsed_args)

        self.assertEqual(['Functions', 'Description'], columns)
        self.assertEqual([('func1', 'Function 1'), ('func2', 'Function 2')],
                         list(data))

    def test_function_list_with_condition_func(self):
        arglist = ['version1', '--with_conditions']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.template_versions.get.return_value = [self.tv1, self.tv2,
                                                   self.tv_with_cf]
        columns, data = self.cmd.take_action(parsed_args)

        self.assertEqual(['Functions', 'Description'], columns)
        self.assertEqual([('func1', 'Function 1'),
                          ('func2', 'Function 2'),
                          ('condition func', 'Condition Function')],
                         list(data))

    def test_function_list_not_found(self):
        arglist = ['bad_version']
        self.template_versions.get.side_effect = exc.HTTPNotFound
        parsed_args = self.check_parser(self.cmd, arglist, [])

        self.assertRaises(exc.CommandError, self.cmd.take_action, parsed_args)


class TestTemplateValidate(TestTemplate):

    template_path = 'heatclient/tests/test_templates/empty.yaml'
    env_path = 'heatclient/tests/unit/var/environment.json'

    defaults = {
        'environment': {},
        'files': {},
        'parameters': {},
        'template': {'heat_template_version': '2013-05-23'}
    }

    def setUp(self):
        super(TestTemplateValidate, self).setUp()
        self.stack_client = self.app.client_manager.orchestration.stacks
        self.stack_client.validate = mock.MagicMock(return_value={})
        self.cmd = template.Validate(self.app, None)

    def test_validate(self):
        arglist = ['-t', self.template_path]
        parsed_args = self.check_parser(self.cmd, arglist, [])
        columns, data = self.cmd.take_action(parsed_args)

        self.stack_client.validate.assert_called_once_with(**self.defaults)
        self.assertEqual([], columns)
        self.assertEqual([], data)

    def test_validate_env(self):
        arglist = ['-t', self.template_path, '-e', self.env_path]
        parsed_args = self.check_parser(self.cmd, arglist, [])
        columns, data = self.cmd.take_action(parsed_args)

        self.assertEqual(1, self.stack_client.validate.call_count)
        args = self.stack_client.validate.call_args[1]
        self.assertEqual(args.get('environment'), {'parameters': {}})
        self.assertIn(self.env_path, args.get('environment_files')[0])
        self.assertEqual([], columns)
        self.assertEqual([], data)

    def test_validate_nested(self):
        arglist = ['-t', self.template_path, '--show-nested']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        columns, data = self.cmd.take_action(parsed_args)
        args = dict(self.defaults)
        args['show_nested'] = True
        self.stack_client.validate.assert_called_once_with(**args)
        self.assertEqual([], columns)
        self.assertEqual([], data)

    def test_validate_parameter(self):
        arglist = ['-t', self.template_path,
                   '--parameter', 'key1=value1',
                   '--parameter', 'key2=value2']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        columns, data = self.cmd.take_action(parsed_args)
        args = dict(self.defaults)
        args['parameters'] = {'key1': 'value1', 'key2': 'value2'}
        self.stack_client.validate.assert_called_once_with(**args)
        self.assertEqual([], columns)
        self.assertEqual([], data)

    def test_validate_ignore_errors(self):
        arglist = ['-t', self.template_path,
                   '--ignore-errors', 'err1,err2']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        columns, data = self.cmd.take_action(parsed_args)
        args = dict(self.defaults)
        args['ignore_errors'] = 'err1,err2'
        self.stack_client.validate.assert_called_once_with(**args)
        self.assertEqual([], columns)
        self.assertEqual([], data)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

import testtools

from heatclient.common import event_utils
from heatclient.v1 import events as hc_ev
from heatclient.v1 import resources as hc_res


class FakeWebSocket(object):

    def __init__(self, events):
        self.events = events

    def recv(self):
        return self.events.pop(0)


class ShellTestEventUtils(testtools.TestCase):
    @staticmethod
    def _mock_resource(resource_id, nested_id=None):
        res_info = {"links": [{"href": "http://heat/foo", "rel": "self"},
                              {"href": "http://heat/foo2", "rel": "resource"}],
                    "logical_resource_id": resource_id,
                    "physical_resource_id": resource_id,
                    "resource_status": "CREATE_COMPLETE",
                    "resource_status_reason": "state changed",
                    "resource_type": "OS::Nested::Server",
                    "updated_time": "2014-01-06T16:14:26Z"}
        if nested_id:
            nested_link = {"href": "http://heat/%s" % nested_id,
                           "rel": "nested"}
            res_info["links"].append(nested_link)
        return hc_res.Resource(manager=None, info=res_info)

    @staticmethod
    def _mock_event(event_id, resource_id,
                    resource_status='CREATE_COMPLETE'):
        ev_info = {"links": [
                   {"href": "http://heat/foo", "rel": "self"},
                   {"href": "http://heat/stacks/astack", "rel": "stack"}],
                   "logical_resource_id": resource_id,
                   "physical_resource_id": resource_id,
                   "resource_name": resource_id,
                   "resource_status": resource_status,
                   "resource_status_reason": "state changed",
                   "event_time": "2014-12-05T14:14:30Z",
                   "id": event_id}
        return hc_ev.Event(manager=None, info=ev_info)

    @staticmethod
    def _mock_stack_event(event_id, stack_name,
                          stack_status='CREATE_COMPLETE'):
        stack_id = 'abcdef'
        ev_info = {"links": [{"href": "http://heat/foo", "rel": "self"},
                             {"href": "http://heat/stacks/%s/%s" % (stack_name,
                                                                    stack_id),
                              "rel": "stack"}],
                   "logical_resource_id": stack_name,
                   "physical_resource_id": stack_id,
                   "resource_name": stack_name,
                   "resource_status": stack_status,
                   "resource_status_reason": "state changed",
                   "event_time": "2014-12-05T14:14:30Z",
                   "id": event_id}
        return hc_ev.Event(manager=None, info=ev_info)

    def test_get_nested_ids(self):
        def list_stub(stack_id):
            return [self._mock_resource('aresource', 'foo3/3id')]
        mock_client = mock.MagicMock()
        mock_client.resources.list.side_effect = list_stub
        ids = event_utils._get_nested_ids(hc=mock_client,
                                          stack_id='astack/123')
        mock_client.resources.list.assert_called_once_with(
            stack_id='astack/123')
        self.assertEqual(['foo3/3id'], ids)

    def test_get_stack_events(self):
        def event_stub(stack_id, argfoo):
            return [self._mock_event('event1', 'aresource')]
        mock_client = mock.MagicMock()
        mock_client.events.list.side_effect = event_stub
        ev_args = {'argfoo': 123}
        evs = event_utils._get_stack_events(hc=mock_client,
                                            stack_id='astack/123',
                                            event_args=ev_args)
        mock_client.events.list.assert_called_once_with(
            stack_id='astack/123', argfoo=123)
        self.assertEqual(1, len(evs))
        self.assertEqual('event1', evs[0].id)
        self.assertEqual('astack', evs[0].stack_name)

    def test_get_nested_events(self):
        resources = {'parent': self._mock_resource('resource1', 'foo/child1'),
                     'foo/child1': self._mock_resource('res_child1',
                                                       'foo/child2'),
                     'foo/child2': self._mock_resource('res_child2',
                                                       'foo/child3'),
                     'foo/child3': self._mock_resource('res_child3',
                                                       'foo/END')}

        def resource_list_stub(stack_id):
            return [resources[stack_id]]
        mock_client = mock.MagicMock()
        mock_client.resources.list.side_effect = resource_list_stub

        events = {'foo/child1': self._mock_event('event1', 'res_child1'),
                  'foo/child2': self._mock_event('event2', 'res_child2'),
                  'foo/child3': self._mock_event('event3', 'res_child3')}

        def event_list_stub(stack_id, argfoo):
            return [events[stack_id]]
        mock_client.events.list.side_effect = event_list_stub

        ev_args = {'argfoo': 123}
        # Check nested_depth=1 (non recursive)..
        evs = event_utils._get_nested_events(hc=mock_client,
                                             nested_depth=1,
                                             stack_id='parent',
                                             event_args=ev_args)

        rsrc_calls = [mock.call(stack_id='parent')]
        mock_client.resources.list.assert_has_calls(rsrc_calls)
        ev_calls = [mock.call(stack_id='foo/child1', argfoo=123)]
        mock_client.events.list.assert_has_calls(ev_calls)
        self.assertEqual(1, len(evs))
        self.assertEqual('event1', evs[0].id)

        # ..and the recursive case via nested_depth=3
        mock_client.resources.list.reset_mock()
        mock_client.events.list.reset_mock()
        evs = event_utils._get_nested_events(hc=mock_client,
                                             nested_depth=3,
                                             stack_id='parent',
                                             event_args=ev_args)

        rsrc_calls = [mock.call(stack_id='parent'),
                      mock.call(stack_id='foo/child1'),
                      mock.call(stack_id='foo/child2')]
        mock_client.resources.list.assert_has_calls(rsrc_calls)
        ev_calls = [mock.call(stack_id='foo/child1', argfoo=123),
                    mock.call(stack_id='foo/child2', argfoo=123),
                    mock.call(stack_id='foo/child3', argfoo=123)]
        mock_client.events.list.assert_has_calls(ev_calls)
        self.assertEqual(3, len(evs))
        self.assertEqual('event1', evs[0].id)
        self.assertEqual('event2', evs[1].id)
        self.assertEqual('event3', evs[2].id)

    @mock.patch('heatclient.common.event_utils.get_events')
    def test_poll_for_events(self, ge):
        ge.side_effect = [[
            self._mock_stack_event('1', 'astack', 'CREATE_IN_PROGRESS'),
            self._mock_event('2', 'res_child1', 'CREATE_IN_PROGRESS'),
            self._mock_event('3', 'res_child2', 'CREATE_IN_PROGRESS'),
            self._mock_event('4', 'res_child3', 'CREATE_IN_PROGRESS')
        ], [
            self._mock_event('5', 'res_child1', 'CREATE_COMPLETE'),
            self._mock_event('6', 'res_child2', 'CREATE_COMPLETE'),
            self._mock_event('7', 'res_child3', 'CREATE_COMPLETE'),
            self._mock_stack_event('8', 'astack', 'CREATE_COMPLETE')
        ]]

        stack_status, msg = event_utils.poll_for_events(
            None, 'astack', action='CREATE', poll_period=0)
        self.assertEqual('CREATE_COMPLETE', stack_status)
        self.assertEqual('\n Stack astack CREATE_COMPLETE \n', msg)
        ge.assert_has_calls([
            mock.call(None, stack_id='astack', nested_depth=0, event_args={
                'sort_dir': 'asc', 'marker': None
            }),
            mock.call(None, stack_id='astack', nested_depth=0, event_args={
                'sort_dir': 'asc', 'marker': '4'
            })
        ])

    @mock.patch('heatclient.common.event_utils.get_events')
    def test_poll_for_events_same_name(self, ge):
        ge.side_effect = [[
            self._mock_stack_event('1', 'mything', 'CREATE_IN_PROGRESS'),
            self._mock_event('2', 'res_child1', 'CREATE_IN_PROGRESS'),
            self._mock_event('3', 'mything', 'CREATE_IN_PROGRESS'),
        ], [
            self._mock_event('4', 'mything', 'CREATE_COMPLETE'),
        ], [
            self._mock_event('5', 'res_child1', 'CREATE_COMPLETE'),
            self._mock_stack_event('6', 'mything', 'CREATE_COMPLETE'),
        ]]

        stack_status, msg = event_utils.poll_for_events(
            None, 'mything', action='CREATE', poll_period=0)
        self.assertEqual('CREATE_COMPLETE', stack_status)
        self.assertEqual('\n Stack mything CREATE_COMPLETE \n', msg)
        ge.assert_has_calls([
            mock.call(None, stack_id='mything', nested_depth=0, event_args={
                'sort_dir': 'asc', 'marker': None
            }),
            mock.call(None, stack_id='mything', nested_depth=0, event_args={
                'sort_dir': 'asc', 'marker': '3'
            }),
            mock.call(None, stack_id='mything', nested_depth=0, event_args={
                'sort_dir': 'asc', 'marker': '4'
            })
        ])

    @mock.patch('heatclient.common.event_utils.get_events')
    def test_poll_for_events_with_marker(self, ge):
        ge.side_effect = [[
            self._mock_event('5', 'res_child1', 'CREATE_COMPLETE'),
            self._mock_event('6', 'res_child2', 'CREATE_COMPLETE'),
            self._mock_event('7', 'res_child3', 'CREATE_COMPLETE'),
            self._mock_stack_event('8', 'astack', 'CREATE_COMPLETE')
        ]]

        stack_status, msg = event_utils.poll_for_events(
            None, 'astack', action='CREATE', poll_period=0, marker='4',
            nested_depth=0)
        self.assertEqual('CREATE_COMPLETE', stack_status)
        self.assertEqual('\n Stack astack CREATE_COMPLETE \n', msg)
        ge.assert_has_calls([
            mock.call(None, stack_id='astack', nested_depth=0, event_args={
                'sort_dir': 'asc', 'marker': '4'
            })
        ])

    @mock.patch('heatclient.common.event_utils.get_events')
    def test_poll_for_events_in_progress_resource(self, ge):
        ge.side_effect = [[
            self._mock_stack_event('1', 'astack', 'CREATE_IN_PROGRESS'),
            self._mock_event('2', 'res_child1', 'CREATE_IN_PROGRESS'),
            self._mock_stack_event('3', 'astack', 'CREATE_COMPLETE')
        ]]

        stack_status, msg = event_utils.poll_for_events(
            None, 'astack', action='CREATE', poll_period=0)
        self.assertEqual('CREATE_COMPLETE', stack_status)
        self.assertEqual('\n Stack astack CREATE_COMPLETE \n', msg)

    @mock.patch('heatclient.common.event_utils.get_events')
    def test_poll_for_events_failed(self, ge):
        ge.side_effect = [[
            self._mock_stack_event('1', 'astack', 'CREATE_IN_PROGRESS'),
            self._mock_event('2', 'res_child1', 'CREATE_IN_PROGRESS'),
            self._mock_event('3', 'res_child2', 'CREATE_IN_PROGRESS'),
            self._mock_event('4', 'res_child3', 'CREATE_IN_PROGRESS')
        ], [
            self._mock_event('5', 'res_child1', 'CREATE_COMPLETE'),
            self._mock_event('6', 'res_child2', 'CREATE_FAILED'),
            self._mock_event('7', 'res_child3', 'CREATE_COMPLETE'),
            self._mock_stack_event('8', 'astack', 'CREATE_FAILED')
        ]]

        stack_status, msg = event_utils.poll_for_events(
            None, 'astack', action='CREATE', poll_period=0)
        self.assertEqual('CREATE_FAILED', stack_status)
        self.assertEqual('\n Stack astack CREATE_FAILED \n', msg)

    @mock.patch('heatclient.common.event_utils.get_events')
    def test_poll_for_events_no_action(self, ge):
        ge.side_effect = [[
            self._mock_stack_event('1', 'astack', 'CREATE_IN_PROGRESS'),
            self._mock_event('2', 'res_child1', 'CREATE_IN_PROGRESS'),
            self._mock_event('3', 'res_child2', 'CREATE_IN_PROGRESS'),
            self._mock_event('4', 'res_child3', 'CREATE_IN_PROGRESS')
        ], [
            self._mock_event('5', 'res_child1', 'CREATE_COMPLETE'),
            self._mock_event('6', 'res_child2', 'CREATE_FAILED'),
            self._mock_event('7', 'res_child3', 'CREATE_COMPLETE'),
            self._mock_stack_event('8', 'astack', 'FOO_FAILED')
        ]]

        stack_status, msg = event_utils.poll_for_events(
            None, 'astack', action=None, poll_period=0)
        self.assertEqual('FOO_FAILED', stack_status)
        self.assertEqual('\n Stack astack FOO_FAILED \n', msg)

    @mock.patch('heatclient.common.event_utils.get_events')
    def test_poll_for_events_stack_get(self, ge):
        mock_client = mock.MagicMock()
        mock_client.stacks.get.return_value.stack_status = 'CREATE_FAILED'

        ge.side_effect = [[
            self._mock_stack_event('1', 'astack', 'CREATE_IN_PROGRESS'),
            self._mock_event('2', 'res_child1', 'CREATE_IN_PROGRESS'),
            self._mock_event('3', 'res_child2', 'CREATE_IN_PROGRESS'),
            self._mock_event('4', 'res_child3', 'CREATE_IN_PROGRESS')
        ], [], []]

        stack_status, msg = event_utils.poll_for_events(
            mock_client, 'astack', action='CREATE', poll_period=0)
        self.assertEqual('CREATE_FAILED', stack_status)
        self.assertEqual('\n Stack astack CREATE_FAILED \n', msg)

    def test_wait_for_events(self):
        ws = FakeWebSocket([
            {'body': {
                'timestamp': '2014-01-06T16:14:26Z',
                'payload': {'resource_action': 'CREATE',
                            'resource_status': 'COMPLETE',
                            'resource_name': 'mystack',
                            'physical_resource_id': 'stackid1',
                            'stack_id': 'stackid1'}}}])
        stack_status, msg = event_utils.wait_for_events(ws, 'mystack')
        self.assertEqual('CREATE_COMPLETE', stack_status)
        self.assertEqual('\n Stack mystack CREATE_COMPLETE \n', msg)

    def test_wait_for_events_failed(self):
        ws = FakeWebSocket([
            {'body': {
                'timestamp': '2014-01-06T16:14:23Z',
                'payload': {'resource_action': 'CREATE',
                            'resource_status': 'IN_PROGRESS',
                            'resource_name': 'mystack',
                            'physical_resource_id': 'stackid1',
                            'stack_id': 'stackid1'}}},
            {'body': {
                'timestamp': '2014-01-06T16:14:26Z',
                'payload': {'resource_action': 'CREATE',
                            'resource_status': 'FAILED',
                            'resource_name': 'mystack',
                            'physical_resource_id': 'stackid1',
                            'stack_id': 'stackid1'}}}])
        stack_status, msg = event_utils.wait_for_events(ws, 'mystack')
        self.assertEqual('CREATE_FAILED', stack_status)
        self.assertEqual('\n Stack mystack CREATE_FAILED \n', msg)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

"""Orchestration v1 Service action implementations"""

import logging

from osc_lib.command import command
from osc_lib import utils


class ListService(command.Lister):
    """List the Heat engines."""

    log = logging.getLogger(__name__ + ".ListService")

    def get_parser(self, prog_name):
        parser = super(ListService, self).get_parser(prog_name)
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)

        heat_client = self.app.client_manager.orchestration
        columns = ['Hostname', 'Binary', 'Engine ID', 'Host',
                   'Topic', 'Updated At', 'Status']
        services = heat_client.services.list()
        return (
            columns,
            (utils.get_item_properties(s, columns) for s in services)
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

import testtools

from heatclient.common import utils
from heatclient.v1 import software_deployments


class SoftwareDeploymentTest(testtools.TestCase):

    def setUp(self):
        super(SoftwareDeploymentTest, self).setUp()
        deployment_id = 'bca6871d-86c0-4aff-b792-58a1f6947b57'
        self.deployment = software_deployments.SoftwareDeployment(
            mock.MagicMock(), info={'id': deployment_id})
        self.deployment_id = deployment_id

    def test_delete(self):
        self.deployment.manager.delete.return_value = None
        self.assertIsNone(self.deployment.delete())
        kwargs = self.deployment.manager.delete.call_args[1]
        self.assertEqual(self.deployment_id, kwargs['deployment_id'])

    def test_update(self):
        self.assertEqual(
            "<SoftwareDeployment {'id': '%s'}>" % self.deployment_id,
            str(self.deployment))
        self.deployment.manager.update.return_value = None
        config_id = 'd00ba4aa-db33-42e1-92f4-2a6469260107'
        self.assertIsNone(self.deployment.update(config_id=config_id))
        kwargs = self.deployment.manager.update.call_args[1]
        self.assertEqual(self.deployment_id, kwargs['deployment_id'])
        self.assertEqual(config_id, kwargs['config_id'])


class SoftwareDeploymentManagerTest(testtools.TestCase):

    def setUp(self):
        super(SoftwareDeploymentManagerTest, self).setUp()
        self.manager = software_deployments.SoftwareDeploymentManager(
            mock.MagicMock())

    def test_list(self):
        server_id = 'fc01f89f-e151-4dc5-9c28-543c0d20ed6a'
        self.manager.client.json_request.return_value = (
            {},
            {'software_deployments': []})
        result = self.manager.list(server_id=server_id)
        self.assertEqual([], result)
        call_args = self.manager.client.get.call_args
        self.assertEqual(
            ('/software_deployments?server_id=%s' % server_id,),
            *call_args)

    @mock.patch.object(utils, 'get_response_body')
    def test_metadata(self, mock_utils):
        server_id = 'fc01f89f-e151-4dc5-9c28-543c0d20ed6a'
        metadata = {
            'group1': [{'foo': 'bar'}],
            'group2': [{'foo': 'bar'}, {'bar': 'baz'}],
        }
        self.manager.client.get.return_value = {}
        mock_utils.return_value = {'metadata': metadata}
        result = self.manager.metadata(server_id=server_id)
        self.assertEqual(metadata, result)
        call_args = self.manager.client.get.call_args
        self.assertEqual(
            '/software_deployments/metadata/%s' % server_id,
            call_args[0][0])

    @mock.patch.object(utils, 'get_response_body')
    def test_get(self, mock_utils):
        deployment_id = 'bca6871d-86c0-4aff-b792-58a1f6947b57'
        config_id = 'd00ba4aa-db33-42e1-92f4-2a6469260107'
        server_id = 'fb322564-7927-473d-8aad-68ae7fbf2abf'
        data = {
            'id': deployment_id,
            'server_id': server_id,
            'input_values': {},
            'output_values': {},
            'action': 'INIT',
            'status': 'COMPLETE',
            'status_reason': None,
            'signal_id': None,
            'config_id': config_id,
            'config': '#!/bin/bash',
            'name': 'config_mysql',
            'group': 'Heat::Shell',
            'inputs': [],
            'outputs': [],
            'options': []}

        self.manager.client.get.return_value = {}
        mock_utils.return_value = {'software_deployment': data}
        result = self.manager.get(deployment_id=deployment_id)
        self.assertEqual(software_deployments.SoftwareDeployment(
            self.manager, data), result)
        call_args = self.manager.client.get.call_args
        self.assertEqual(
            ('/software_deployments/%s' % deployment_id,), *call_args)

    @mock.patch.object(utils, 'get_response_body')
    def test_create(self, mock_utils):
        deployment_id = 'bca6871d-86c0-4aff-b792-58a1f6947b57'
        config_id = 'd00ba4aa-db33-42e1-92f4-2a6469260107'
        server_id = 'fb322564-7927-473d-8aad-68ae7fbf2abf'
        body = {
            'server_id': server_id,
            'input_values': {},
            'action': 'INIT',
            'status': 'COMPLETE',
            'status_reason': None,
            'signal_id': None,
            'config_id': config_id}
        data = body.copy()
        data['id'] = deployment_id
        self.manager.client.post.return_value = {}
        mock_utils.return_value = {'software_deployment': data}
        result = self.manager.create(**body)
        self.assertEqual(software_deployments.SoftwareDeployment(
            self.manager, data), result)
        args, kwargs = self.manager.client.post.call_args
        self.assertEqual('/software_deployments', args[0])
        self.assertEqual({'data': body}, kwargs)

    def test_delete(self):
        deployment_id = 'bca6871d-86c0-4aff-b792-58a1f6947b57'
        self.manager.delete(deployment_id)
        call_args = self.manager.client.delete.call_args
        self.assertEqual(
            ('/software_deployments/%s' % deployment_id,), *call_args)

    @mock.patch.object(utils, 'get_response_body')
    def test_update(self, mock_utils):
        deployment_id = 'bca6871d-86c0-4aff-b792-58a1f6947b57'
        config_id = 'd00ba4aa-db33-42e1-92f4-2a6469260107'
        server_id = 'fb322564-7927-473d-8aad-68ae7fbf2abf'
        body = {
            'server_id': server_id,
            'input_values': {},
            'action': 'DEPLOYED',
            'status': 'COMPLETE',
            'status_reason': None,
            'signal_id': None,
            'config_id': config_id}
        data = body.copy()
        data['id'] = deployment_id
        self.manager.client.put.return_value = {}
        mock_utils.return_value = {'software_deployment': data}
        result = self.manager.update(deployment_id, **body)
        self.assertEqual(software_deployments.SoftwareDeployment(
            self.manager, data), result)
        args, kwargs = self.manager.client.put.call_args
        self.assertEqual('/software_deployments/%s' % deployment_id, args[0])
        self.assertEqual({'data': body}, kwargs)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils as utils

from heatclient.tests.functional import config
from heatclient.tests.functional.osc.v1 import base


class OpenStackClientStackTest(base.OpenStackClientTestBase):
    """Basic stack operation tests for Openstack CLI client heat plugin.

    Basic smoke test for the openstack CLI stack commands.
    """

    def setUp(self):
        super(OpenStackClientStackTest, self).setUp()
        self.stack_name = utils.rand_name(name='test-stack')

    def _stack_create_minimal(self, from_url=False):
        if from_url:
            template = config.HEAT_MINIMAL_HOT_TEMPLATE_URL
        else:
            template = self.get_template_path('heat_minimal_hot.yaml')
        parameters = ['test_client_name=test_client_name']
        return self._stack_create(self.stack_name, template=template,
                                  parameters=parameters)

    def test_stack_create_minimal_from_file(self):
        stack = self._stack_create_minimal()
        self.assertEqual(self.stack_name, stack['stack_name'])
        self.assertEqual("CREATE_COMPLETE", stack['stack_status'])

    def test_stack_create_minimal_from_url(self):
        stack = self._stack_create_minimal(from_url=True)
        self.assertEqual(self.stack_name, stack['stack_name'])
        self.assertEqual("CREATE_COMPLETE", stack['stack_status'])

    def test_stack_suspend_resume(self):
        stack = self._stack_create_minimal()
        stack = self._stack_suspend(stack['id'])
        self.assertEqual(self.stack_name, stack['stack_name'])
        self.assertEqual("SUSPEND_COMPLETE", stack['stack_status'])
        stack = self._stack_resume(stack['id'])
        self.assertEqual(self.stack_name, stack['stack_name'])
        self.assertEqual("RESUME_COMPLETE", stack['stack_status'])

    def test_stack_snapshot_create_restore(self):
        snapshot_name = utils.rand_name(name='test-stack-snapshot')
        stack = self._stack_create_minimal()
        snapshot = self._stack_snapshot_create(stack['id'], snapshot_name)
        self.assertEqual(snapshot_name, snapshot['name'])
        self._stack_snapshot_restore(stack['id'], snapshot['id'])

    def test_stack_delete(self):
        stack = self._stack_create_minimal()
        self._stack_delete(stack['id'])
        stacks_raw = self.openstack('stack list')
        self.assertNotIn(stack['id'], stacks_raw)

    def test_stack_snapshot_delete(self):
        snapshot_name = utils.rand_name(name='test-stack-snapshot')
        stack = self._stack_create_minimal()
        snapshot = self._stack_snapshot_create(stack['id'], snapshot_name)
        self._stack_snapshot_delete(stack['id'], snapshot['id'])
        stacks_raw = self.openstack(
            'stack snapshot list' + ' ' + self.stack_name)
        self.assertNotIn(snapshot['id'], stacks_raw)

    def test_stack_show(self):
        stack = self._stack_create_minimal()
        stack_info = self._stack_show(stack['id'])
        stack_props = {k: v for k, v in stack_info.items()
                       if k in stack.keys()}
        self.assertEqual(stack, stack_props)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

"""Orchestration v1 Stack Snapshot implementations."""

import logging
import sys

from osc_lib.command import command
from osc_lib import exceptions as exc
from osc_lib.i18n import _
from osc_lib import utils

from heatclient.common import format_utils
from heatclient import exc as heat_exc


class ListSnapshot(command.Lister):
    """List stack snapshots."""

    log = logging.getLogger(__name__ + ".ListSnapshot")

    def get_parser(self, prog_name):
        parser = super(ListSnapshot, self).get_parser(prog_name)
        parser.add_argument(
            'stack',
            metavar='<stack>',
            help=_('Name or ID of stack containing the snapshots')
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)', parsed_args)
        heat_client = self.app.client_manager.orchestration
        return self._list_snapshot(heat_client, parsed_args)

    def _list_snapshot(self, heat_client, parsed_args):
        fields = {'stack_id': parsed_args.stack}
        try:
            snapshots = heat_client.stacks.snapshot_list(**fields)
        except heat_exc.HTTPNotFound:
            raise exc.CommandError(_('Stack not found: %s') %
                                   parsed_args.stack)

        columns = ['id', 'name', 'status', 'status_reason', 'creation_time']
        return (
            columns,
            (utils.get_dict_properties(s, columns)
             for s in snapshots['snapshots'])
        )


class ShowSnapshot(format_utils.YamlFormat):
    """Show stack snapshot."""

    log = logging.getLogger(__name__ + ".ShowSnapshot")

    def get_parser(self, prog_name):
        parser = super(ShowSnapshot, self).get_parser(prog_name)
        parser.add_argument(
            'stack',
            metavar='<stack>',
            help=_('Name or ID of stack containing the snapshot')
        )
        parser.add_argument(
            'snapshot',
            metavar='<snapshot>',
            help=_('ID of the snapshot to show')
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)', parsed_args)
        heat_client = self.app.client_manager.orchestration
        return self._show_snapshot(heat_client, parsed_args.stack,
                                   parsed_args.snapshot)

    def _show_snapshot(self, heat_client, stack_id, snapshot_id):
        try:
            data = heat_client.stacks.snapshot_show(stack_id, snapshot_id)
        except heat_exc.HTTPNotFound:
            raise exc.CommandError(_('Snapshot ID <%(snapshot_id)s> not found '
                                     'for stack <%(stack_id)s>')
                                   % {'snapshot_id': snapshot_id,
                                      'stack_id': stack_id})

        rows = list(data.values())
        columns = list(data.keys())
        return columns, rows


class RestoreSnapshot(command.Command):
    """Restore stack snapshot"""

    log = logging.getLogger(__name__ + ".RestoreSnapshot")

    def get_parser(self, prog_name):
        parser = super(RestoreSnapshot, self).get_parser(prog_name)
        parser.add_argument(
            'stack',
            metavar='<stack>',
            help=_('Name or ID of stack containing the snapshot')
        )
        parser.add_argument(
            'snapshot',
            metavar='<snapshot>',
            help=_('ID of the snapshot to restore')
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)', parsed_args)
        heat_client = self.app.client_manager.orchestration
        return self._restore_snapshot(heat_client, parsed_args)

    def _restore_snapshot(self, heat_client, parsed_args):
        fields = {'stack_id': parsed_args.stack,
                  'snapshot_id': parsed_args.snapshot}
        try:
            heat_client.stacks.restore(**fields)
        except heat_exc.HTTPNotFound:
            raise exc.CommandError(_('Stack %(stack)s or '
                                     'snapshot %(snapshot)s not found.') %
                                   {'stack': parsed_args.stack,
                                    'snapshot': parsed_args.snapshot})


class CreateSnapshot(command.ShowOne):
    """Create stack snapshot."""

    log = logging.getLogger(__name__ + ".CreateSnapshot")

    def get_parser(self, prog_name):
        parser = super(CreateSnapshot, self).get_parser(prog_name)
        parser.add_argument(
            'stack',
            metavar='<stack>',
            help=_('Name or ID of stack')
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('Name of snapshot')
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)', parsed_args)
        heat_client = self.app.client_manager.orchestration

        try:
            data = heat_client.stacks.snapshot(parsed_args.stack,
                                               parsed_args.name)
        except heat_exc.HTTPNotFound:
            raise exc.CommandError(_('Stack not found: %s')
                                   % parsed_args.stack)

        columns = [
            'ID',
            'name',
            'status',
            'status_reason',
            'data',
            'creation_time'
        ]
        return (columns, utils.get_dict_properties(data, columns))


class DeleteSnapshot(command.Command):
    """Delete stack snapshot."""
    log = logging.getLogger(__name__ + ".DeleteSnapshot")

    def get_parser(self, prog_name):
        parser = super(DeleteSnapshot, self).get_parser(prog_name)
        parser.add_argument(
            'stack',
            metavar='<stack>',
            help=_('Name or ID of stack')
        )
        parser.add_argument(
            'snapshot',
            metavar='<snapshot>',
            help=_('ID of stack snapshot')
        )
        parser.add_argument(
            '-y', '--yes',
            action='store_true',
            help=_('Skip yes/no prompt (assume yes)')
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)', parsed_args)
        heat_client = self.app.client_manager.orchestration
        msg = ('User did not confirm snapshot delete '
               '%sso taking no action.')
        try:
            if not parsed_args.yes and sys.stdin.isatty():
                sys.stdout.write(
                    _('Are you sure you want to delete the snapshot of this '
                      'stack [Y/N]?'))
                prompt_response = sys.stdin.readline().lower()
                if not prompt_response.startswith('y'):
                    self.log.info(msg, '')
                    return
        except KeyboardInterrupt:  # ctrl-c
            self.log.info(msg, '(ctrl-c) ')
            return
        except EOFError:  # ctrl-d
            self.log.info(msg, '(ctrl-d) ')
            return

        try:
            heat_client.stacks.snapshot_delete(parsed_args.stack,
                                               parsed_args.snapshot)
        except heat_exc.HTTPNotFound:
            raise exc.CommandError(_('Snapshot ID <%(snapshot_id)s> not found '
                                     'for stack <%(stack_id)s>')
                                   % {'snapshot_id': parsed_args.snapshot,
                                      'stack_id': parsed_args.stack})



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

import testscenarios
import testtools
import yaml

from heatclient.common import environment_format

load_tests = testscenarios.load_tests_apply_scenarios


class YamlEnvironmentTest(testtools.TestCase):

    def test_minimal_yaml(self):
        yaml1 = ''
        yaml2 = '''
parameter_defaults: {}
encrypted_param_names: []
parameters: {}
resource_registry: {}
event_sinks: []
'''
        tpl1 = environment_format.parse(yaml1)
        environment_format.default_for_missing(tpl1)
        tpl2 = environment_format.parse(yaml2)
        self.assertEqual(tpl2, tpl1)

    def test_param_valid_strategy_section(self):
        yaml1 = ''
        yaml2 = '''
parameters: {}
encrypted_param_names: []
parameter_defaults: {}
parameter_merge_strategies: {}
event_sinks: []
resource_registry: {}
'''
        tpl1 = environment_format.parse(yaml1)
        environment_format.default_for_missing(tpl1)
        tpl2 = environment_format.parse(yaml2)
        self.assertNotEqual(tpl1, tpl2)

    def test_wrong_sections(self):
        env = '''
parameters: {}
resource_regis: {}
'''
        self.assertRaises(ValueError, environment_format.parse, env)

    def test_bad_yaml(self):
        env = '''
parameters: }
'''
        self.assertRaises(ValueError, environment_format.parse, env)

    def test_parse_string_environment(self):
        env = 'just string'
        expect = 'The environment is not a valid YAML mapping data type.'
        e = self.assertRaises(ValueError, environment_format.parse, env)
        self.assertIn(expect, str(e))

    def test_parse_document(self):
        env = '["foo", "bar"]'
        expect = 'The environment is not a valid YAML mapping data type.'
        e = self.assertRaises(ValueError, environment_format.parse, env)
        self.assertIn(expect, str(e))


class YamlParseExceptions(testtools.TestCase):

    scenarios = [
        ('scanner', dict(raised_exception=yaml.scanner.ScannerError())),
        ('parser', dict(raised_exception=yaml.parser.ParserError())),
        ('reader',
         dict(raised_exception=yaml.reader.ReaderError('', '', '', '', ''))),
    ]

    def test_parse_to_value_exception(self):
        text = 'not important'

        with mock.patch.object(yaml, 'load') as yaml_loader:
            yaml_loader.side_effect = self.raised_exception

            self.assertRaises(ValueError,
                              environment_format.parse, text)


class DetailedYAMLParseExceptions(testtools.TestCase):

    def test_parse_to_value_exception(self):
        yaml = """not important
but very:
  - incorrect
"""
        ex = self.assertRaises(ValueError, environment_format.parse, yaml)
        self.assertIn('but very:\n            ^', str(ex))



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
#   Copyright 2015 IBM Corp.

import logging
import time

from cliff.formatters import base
from osc_lib.command import command
from osc_lib import utils

from heatclient._i18n import _
from heatclient.common import event_utils
from heatclient.common import utils as heat_utils
from heatclient import exc
from heatclient.osc.v1 import common


class ShowEvent(command.ShowOne):
    """Show event details."""

    log = logging.getLogger(__name__ + '.ShowEvent')

    def get_parser(self, prog_name):
        parser = super(ShowEvent, self).get_parser(prog_name)
        parser.add_argument(
            'stack',
            metavar='<stack>',
            help=_('Name or ID of stack to show events for')
        )
        parser.add_argument(
            'resource',
            metavar='<resource>',
            help=_('Name of the resource event belongs to')
        )
        parser.add_argument(
            'event',
            metavar='<event>',
            help=_('ID of event to display details for')
        )

        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)', parsed_args)

        client = self.app.client_manager.orchestration

        fields = {
            'stack_id': parsed_args.stack,
            'resource_name': parsed_args.resource,
            'event_id': parsed_args.event
        }

        try:
            client.stacks.get(parsed_args.stack)
            client.resources.get(parsed_args.stack, parsed_args.resource)
            event = client.events.get(**fields)
        except exc.HTTPNotFound as ex:
            raise exc.CommandError(str(ex))

        formatters = {
            'links': common.LinkColumn,
            'resource_properties': common.JsonColumn
        }

        columns = []
        for key in event.to_dict():
            columns.append(key)

        return columns, utils.get_item_properties(event, columns,
                                                  formatters=formatters)


class ListEvent(command.Lister):
    """List events."""

    log = logging.getLogger(__name__ + '.ListEvent')

    @property
    def formatter_default(self):
        return 'log'

    @property
    def formatter_namespace(self):
        return 'heatclient.event.formatter.list'

    def get_parser(self, prog_name):
        parser = super(ListEvent, self).get_parser(prog_name)
        parser.add_argument(
            'stack',
            metavar='<stack>',
            help=_('Name or ID of stack to show events for')
        )
        parser.add_argument(
            '--resource',
            metavar='<resource>',
            help=_('Name of resource to show events for. Note: this cannot '
                   'be specified with --nested-depth')
        )
        parser.add_argument(
            '--filter',
            metavar='<key=value>',
            action='append',
            help=_('Filter parameters to apply on returned events')
        )
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            type=int,
            help=_('Limit the number of events returned')
        )
        parser.add_argument(
            '--marker',
            metavar='<id>',
            help=_('Only return events that appear after the given ID')
        )
        parser.add_argument(
            '--nested-depth',
            metavar='<depth>',
            type=int,
            help=_('Depth of nested stacks from which to display events. '
                   'Note: this cannot be specified with --resource')
        )
        parser.add_argument(
            '--sort',
            metavar='<key>[:<direction>]',
            action='append',
            help=_('Sort output by selected keys and directions (asc or desc) '
                   '(default: asc). Specify multiple times to sort on '
                   'multiple keys. Sort key can be: '
                   '"event_time" (default), "resource_name", "links", '
                   '"logical_resource_id", "resource_status", '
                   '"resource_status_reason", "physical_resource_id", or '
                   '"id". You can leave the key empty and specify ":desc" '
                   'for sorting by reverse time.')
        )
        parser.add_argument(
            '--follow',
            action='store_true',
            help=_('Print events until process is halted')
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug('take_action(%s)', parsed_args)

        client = self.app.client_manager.orchestration

        columns = ['id', 'resource_status', 'resource_status_reason',
                   'event_time', 'physical_resource_id']

        kwargs = {
            'resource_name': parsed_args.resource,
            'filters': heat_utils.format_parameters(parsed_args.filter),
            'sort_dir': 'asc'
        }

        if parsed_args.resource and parsed_args.nested_depth:
            msg = _('--nested-depth cannot be specified with --resource')
            raise exc.CommandError(msg)

        if parsed_args.nested_depth:
            columns.append('stack_name')
            nested_depth = parsed_args.nested_depth
        else:
            nested_depth = 0

        if parsed_args.sort:
            sorts = []
            sort_keys = []
            for sort in parsed_args.sort:
                if sort.startswith(":"):
                    sorts.append(":".join(["event_time", sort.lstrip(":")]))
                else:
                    sorts.append(sort)
                    sort_keys.append(sort.split(":")[0])
            kwargs['sort_keys'] = sort_keys

            if ":" in parsed_args.sort[0]:
                kwargs['sort_dir'] = parsed_args.sort[0].split(":")[1]

        if parsed_args.follow:
            if parsed_args.formatter != 'log':
                msg = _('--follow can only be specified with --format log')
                raise exc.CommandError(msg)

            marker = parsed_args.marker
            try:
                event_log_context = heat_utils.EventLogContext()
                while True:
                    events = event_utils.get_events(
                        client,
                        stack_id=parsed_args.stack,
                        event_args=kwargs,
                        nested_depth=nested_depth,
                        marker=marker)
                    if events:
                        marker = getattr(events[-1], 'id', None)
                        events_log = heat_utils.event_log_formatter(
                            events, event_log_context)
                        self.app.stdout.write(events_log)
                        self.app.stdout.write('\n')
                    time.sleep(5)
                    # this loop never exits
            except (KeyboardInterrupt, EOFError):  # ctrl-c, ctrl-d
                return [], []

        events = event_utils.get_events(
            client, stack_id=parsed_args.stack, event_args=kwargs,
            nested_depth=nested_depth, marker=parsed_args.marker,
            limit=parsed_args.limit)

        if parsed_args.sort:
            events = utils.sort_items(events, ','.join(sorts))

        if parsed_args.formatter == 'log':
            return [], events

        if len(events):
            if hasattr(events[0], 'resource_name'):
                columns.insert(0, 'resource_name')
                columns.append('logical_resource_id')
            else:
                columns.insert(0, 'logical_resource_id')

        return (
            columns,
            (utils.get_item_properties(s, columns) for s in events)
        )


class LogFormatter(base.ListFormatter):
    """A formatter which prints event objects in a log style"""

    def add_argument_group(self, parser):
        pass

    def emit_list(self, column_names, data, stdout, parsed_args):
        stdout.write(heat_utils.event_log_formatter(data))
        stdout.write('\n')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from urllib import parse

from heatclient.common import base
from heatclient.common import utils


class SoftwareConfig(base.Resource):
    def __repr__(self):
        return "<SoftwareConfig %s>" % self._info

    def delete(self):
        return self.manager.delete(config_id=self.id)

    def data(self, **kwargs):
        return self.manager.data(self, **kwargs)


class SoftwareConfigManager(base.BaseManager):
    resource_class = SoftwareConfig

    def list(self, **kwargs):
        """Get a list of software configs.

        :rtype: list of :class:`SoftwareConfig`
        """
        qparams = {}

        for opt, val in kwargs.items():
            if val:
                qparams[opt] = val

        # Transform the dict to a sequence of two-element tuples in fixed
        # order, then the encoded string will be consistent in Python 2&3.
        if qparams:
            new_qparams = sorted(qparams.items(), key=lambda x: x[0])
            query_string = "?%s" % parse.urlencode(new_qparams)
        else:
            query_string = ""
        url = '/software_configs%s' % query_string
        return self._list(url, "software_configs")

    def get(self, config_id):
        """Get the details for a specific software config.

        :param config_id: ID of the software config
        """
        resp = self.client.get('/software_configs/%s' % config_id)
        body = utils.get_response_body(resp)
        return SoftwareConfig(self, body.get('software_config'))

    def create(self, **kwargs):
        """Create a software config."""
        resp = self.client.post('/software_configs',
                                data=kwargs)
        body = utils.get_response_body(resp)
        return SoftwareConfig(self, body.get('software_config'))

    def delete(self, config_id):
        """Delete a software config."""
        self._delete("/software_configs/%s" % config_id)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

from unittest import mock

from osc_lib import exceptions as exc
import yaml

from heatclient import exc as heat_exc
from heatclient.osc.v1 import software_config
from heatclient.tests.unit.osc.v1 import fakes as orchestration_fakes
from heatclient.v1 import software_configs


class TestConfig(orchestration_fakes.TestOrchestrationv1):
    def setUp(self):
        super(TestConfig, self).setUp()
        self.mock_client = self.app.client_manager.orchestration


class TestDeleteConfig(TestConfig):

    def setUp(self):
        super(TestDeleteConfig, self).setUp()
        self.cmd = software_config.DeleteConfig(self.app, None)
        self.mock_delete = self.mock_client.software_configs.delete

    def test_config_delete(self):
        arglist = ['id_123']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.cmd.take_action(parsed_args)
        self.mock_delete.assert_called_with(
            config_id='id_123')

    def test_config_delete_multi(self):
        arglist = ['id_123', 'id_456']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.cmd.take_action(parsed_args)
        self.mock_delete.assert_has_calls(
            [mock.call(config_id='id_123'),
             mock.call(config_id='id_456')])

    def test_config_delete_not_found(self):
        arglist = ['id_123', 'id_456', 'id_789']
        self.mock_client.software_configs.delete.side_effect = [
            None, heat_exc.HTTPNotFound, None]
        parsed_args = self.check_parser(self.cmd, arglist, [])
        error = self.assertRaises(exc.CommandError, self.cmd.take_action,
                                  parsed_args)
        self.mock_delete.assert_has_calls(
            [mock.call(config_id='id_123'),
             mock.call(config_id='id_456'),
             mock.call(config_id='id_789')])
        self.assertEqual('Unable to delete 1 of the 3 software configs.',
                         str(error))


class TestListConfig(TestConfig):

    def setUp(self):
        super(TestListConfig, self).setUp()
        self.cmd = software_config.ListConfig(self.app, None)
        self.mock_client.software_configs.list.return_value = [
            software_configs.SoftwareConfig(None, {})]

    def test_config_list(self):
        arglist = []
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.cmd.take_action(parsed_args)
        self.mock_client.software_configs.list.assert_called_once_with()

    def test_config_list_limit(self):
        arglist = ['--limit', '3']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.cmd.take_action(parsed_args)
        self.mock_client.software_configs.list.assert_called_with(limit=3)

    def test_config_list_marker(self):
        arglist = ['--marker', 'id123']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.cmd.take_action(parsed_args)
        self.mock_client.software_configs.list.assert_called_with(
            marker='id123')


class TestCreateConfig(TestConfig):

    def setUp(self):
        super(TestCreateConfig, self).setUp()
        self.cmd = software_config.CreateConfig(self.app, None)
        self.mock_client.software_configs.create.return_value = \
            software_configs.SoftwareConfig(None, {})

    def test_config_create(self):
        properties = {
            'config': '',
            'group': 'Heat::Ungrouped',
            'name': 'test',
            'options': {},
            'inputs': [],
            'outputs': []
        }
        arglist = ['test']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        columns, rows = self.cmd.take_action(parsed_args)
        self.mock_client.stacks.validate.assert_called_with(**{
            'template': {
                'heat_template_version': '2013-05-23',
                'resources': {
                    'test': {
                        'type': 'OS::Heat::SoftwareConfig',
                        'properties': properties}}}})
        self.mock_client.software_configs.create.assert_called_with(
            **properties)

    def test_config_create_group(self):
        properties = {
            'config': '',
            'group': 'group',
            'name': 'test',
            'options': {},
            'inputs': [],
            'outputs': []
        }
        arglist = ['test', '--group', 'group']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        columns, rows = self.cmd.take_action(parsed_args)
        self.mock_client.stacks.validate.assert_called_with(**{
            'template': {
                'heat_template_version': '2013-05-23',
                'resources': {
                    'test': {
                        'type': 'OS::Heat::SoftwareConfig',
                        'properties': properties}}}})
        self.mock_client.software_configs.create.assert_called_with(
            **properties)

    @mock.patch('urllib.request.urlopen')
    def test_config_create_config_file(self, urlopen):
        properties = {
            'config': 'config',
            'group': 'Heat::Ungrouped',
            'name': 'test',
            'options': {},
            'inputs': [],
            'outputs': []
        }
        data = mock.Mock()
        data.read.side_effect = ['config']
        urlopen.return_value = data

        arglist = ['test', '--config-file', 'config_file']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        columns, rows = self.cmd.take_action(parsed_args)
        self.mock_client.stacks.validate.assert_called_with(**{
            'template': {
                'heat_template_version': '2013-05-23',
                'resources': {
                    'test': {
                        'type': 'OS::Heat::SoftwareConfig',
                        'properties': properties}}}})
        self.mock_client.software_configs.create.assert_called_with(
            **properties)

    @mock.patch('urllib.request.urlopen')
    def test_config_create_definition_file(self, urlopen):
        definition = {
            'inputs': [
                {'name': 'input'},
            ],
            'outputs': [
                {'name': 'output'}
            ],
            'options': {'option': 'value'}
        }

        properties = {
            'config': '',
            'group': 'Heat::Ungrouped',
            'name': 'test'
        }
        properties.update(definition)

        data = mock.Mock()
        data.read.side_effect = [yaml.safe_dump(definition)]
        urlopen.return_value = data

        arglist = ['test', '--definition-file', 'definition-file']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        columns, rows = self.cmd.take_action(parsed_args)
        self.mock_client.stacks.validate.assert_called_with(**{
            'template': {
                'heat_template_version': '2013-05-23',
                'resources': {
                    'test': {
                        'type': 'OS::Heat::SoftwareConfig',
                        'properties': properties}}}})
        self.mock_client.software_configs.create.assert_called_with(
            **properties)


class TestConfigShow(TestConfig):
    columns = (
        'id',
        'name',
        'group',
        'config',
        'inputs',
        'outputs',
        'options',
        'creation_time',
    )

    data = (
        '96dfee3f-27b7-42ae-a03e-966226871ae6',
        'test',
        'Heat::Ungrouped',
        '',
        [],
        [],
        {},
        '2015-12-09T11:55:06',
    )

    response = dict(zip(columns, data))

    def setUp(self):
        super(TestConfigShow, self).setUp()
        self.cmd = software_config.ShowConfig(self.app, None)
        self.mock_client.software_configs.get.return_value = \
            software_configs.SoftwareConfig(None, self.response)

    def test_config_show(self):
        arglist = ['96dfee3f-27b7-42ae-a03e-966226871ae6']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        columns, data = self.cmd.take_action(parsed_args)
        self.mock_client.software_configs.get.assert_called_with(**{
            'config_id': '96dfee3f-27b7-42ae-a03e-966226871ae6',
        })
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_config_show_config_only(self):
        arglist = ['--config-only', '96dfee3f-27b7-42ae-a03e-966226871ae6']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        columns, data = self.cmd.take_action(parsed_args)
        self.mock_client.software_configs.get.assert_called_with(**{
            'config_id': '96dfee3f-27b7-42ae-a03e-966226871ae6',
        })
        self.assertIsNone(columns)
        self.assertIsNone(data)

    def test_config_show_not_found(self):
        arglist = ['96dfee3f-27b7-42ae-a03e-966226871ae6']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.mock_client.software_configs.get.side_effect = \
            heat_exc.HTTPNotFound()
        self.assertRaises(
            exc.CommandError,
            self.cmd.take_action,
            parsed_args)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_utils import encodeutils
from urllib import parse

from heatclient.common import base
from heatclient.common import utils
from heatclient.v1 import stacks

DEFAULT_PAGE_SIZE = 20


class Resource(base.Resource):
    def __repr__(self):
        return "<Resource %s>" % self._info

    def update(self, **fields):
        self.manager.update(self, **fields)

    def delete(self):
        return self.manager.delete(self)

    def data(self, **kwargs):
        return self.manager.data(self, **kwargs)

    @property
    def stack_name(self):
        if not hasattr(self, 'links'):
            return
        for link in self.links:
            if link['rel'] == 'stack':
                return link['href'].split('/')[-2]


class ResourceManager(stacks.StackChildManager):
    resource_class = Resource

    def list(self, stack_id, **kwargs):
        """Get a list of resources.

        :rtype: list of :class:`Resource`
        """
        params = {}

        if 'filters' in kwargs:
            filters = kwargs.pop('filters')
            params.update(filters)

        for key, value in kwargs.items():
            if value:
                params[key] = value

        url = '/stacks/%s/resources' % stack_id
        if params:
            url += '?%s' % parse.urlencode(params, True)

        return self._list(url, "resources")

    def get(self, stack_id, resource_name, with_attr=None):
        """Get the details for a specific resource.

        :param stack_id: ID or name of stack containing the resource
        :param resource_name: ID of resource to get the details for
        :param with_attr: Attributes to show
        """
        stack_id = self._resolve_stack_id(stack_id)
        url_str = '/stacks/%s/resources/%s' % (
                  parse.quote(stack_id),
                  parse.quote(encodeutils.safe_encode(resource_name)))
        if with_attr:
            params = {'with_attr': with_attr}
            url_str += '?%s' % parse.urlencode(params, True)

        resp = self.client.get(url_str)
        body = utils.get_response_body(resp)
        return Resource(self, body.get('resource'))

    def metadata(self, stack_id, resource_name):
        """Get the metadata for a specific resource.

        :param stack_id: ID or name of stack containing the resource
        :param resource_name: ID of resource to get metadata for
        """
        stack_id = self._resolve_stack_id(stack_id)
        url_str = '/stacks/%s/resources/%s/metadata' % (
                  parse.quote(stack_id),
                  parse.quote(encodeutils.safe_encode(resource_name)))
        resp = self.client.get(url_str)
        body = utils.get_response_body(resp)
        return body.get('metadata')

    def signal(self, stack_id, resource_name, data=None):
        """Signal a specific resource.

        :param stack_id: ID or name of stack containing the resource
        :param resource_name: ID of resource to send signal to
        """
        stack_id = self._resolve_stack_id(stack_id)
        url_str = '/stacks/%s/resources/%s/signal' % (
                  parse.quote(stack_id),
                  parse.quote(encodeutils.safe_encode(resource_name)))
        resp = self.client.post(url_str, data=data)
        body = utils.get_response_body(resp)
        return body

    def mark_unhealthy(self, stack_id, resource_name,
                       mark_unhealthy, resource_status_reason):
        """Mark a resource as healthy or unhealthy.

        :param stack_id: ID or name of stack containing the resource
        :param resource_name: ID of resource
        :param mark_unhealthy: Mark resource unhealthy if set to True
        :param resource_status_reason: Reason for resource status change.
        """
        stack_id = self._resolve_stack_id(stack_id)
        url_str = '/stacks/%s/resources/%s' % (
                  parse.quote(stack_id),
                  parse.quote(encodeutils.safe_encode(resource_name)))
        resp = self.client.patch(
            url_str,
            data={"mark_unhealthy": mark_unhealthy,
                  "resource_status_reason": resource_status_reason})

        body = utils.get_response_body(resp)
        return body

    def generate_template(self, resource_name):
        """Deprecated in favor of generate_template in ResourceTypeManager."""

        url_str = '/resource_types/%s/template' % (
                  parse.quote(encodeutils.safe_encode(resource_name)))
        resp = self.client.get(url_str)
        body = utils.get_response_body(resp)
        return body



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_serialization import jsonutils


class FakeRaw(object):
    version = 110


class FakeHTTPResponse(object):

    version = 1.1

    def __init__(self, status_code, reason, headers, content):
        self.headers = headers
        self.content = content
        self.status_code = status_code
        self.reason = reason
        self.raw = FakeRaw()

    def getheader(self, name, default=None):
        return self.headers.get(name, default)

    def getheaders(self):
        return self.headers.items()

    def read(self, amt=None):
        b = self.content
        self.content = None
        return b

    def iter_content(self, chunksize):
        return self.content

    def json(self):
        return jsonutils.loads(self.content)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

import testscenarios
import testtools
import yaml

from heatclient.common import template_format


load_tests = testscenarios.load_tests_apply_scenarios


class YamlParseExceptions(testtools.TestCase):

    scenarios = [
        ('scanner', dict(raised_exception=yaml.scanner.ScannerError())),
        ('parser', dict(raised_exception=yaml.parser.ParserError())),
        ('reader',
         dict(raised_exception=yaml.reader.ReaderError('', '', '', '', ''))),
    ]

    def test_parse_to_value_exception(self):
        text = 'not important'

        with mock.patch.object(yaml, 'load') as yaml_loader:
            yaml_loader.side_effect = self.raised_exception

            self.assertRaises(ValueError,
                              template_format.parse, text)

    def test_parse_no_version_format(self):
        yaml = ''
        self.assertRaises(ValueError, template_format.parse, yaml)
        yaml2 = '''Parameters: {}
Mappings: {}
Resources: {}
Outputs: {}
'''
        self.assertRaises(ValueError, template_format.parse, yaml2)


class DetailedYAMLParseExceptions(testtools.TestCase):

    def test_parse_to_value_exception(self):
        yaml = """not important
but very:
  - incorrect
"""
        ex = self.assertRaises(ValueError, template_format.parse, yaml)
        self.assertIn('but very:\n            ^', str(ex))



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from heatclient import exc
from heatclient.v1 import services


class ManageServiceTest(testtools.TestCase):
    def setUp(self):
        super(ManageServiceTest, self).setUp()

    def test_service_list(self):
        class FakeResponse(object):
            def json(self):
                return {'services': []}

        class FakeClient(object):
            def get(self, *args, **kwargs):
                assert args[0] == ('/services')
                return FakeResponse()

        manager = services.ServiceManager(FakeClient())
        self.assertEqual([], manager.list())

    def test_service_list_403(self):
        class FakeClient403(object):

            def get(self, *args, **kwargs):
                assert args[0] == ('/services')
                raise exc.HTTPForbidden()

        manager = services.ServiceManager(FakeClient403())
        self.assertRaises(exc.HTTPForbidden,
                          manager.list)

    def test_service_list_503(self):
        class FakeClient503(object):
            def get(self, *args, **kwargs):
                assert args[0] == ('/services')
                raise exc.HTTPServiceUnavailable()

        manager = services.ServiceManager(FakeClient503())
        self.assertRaises(exc.HTTPServiceUnavailable,
                          manager.list)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import fnmatch
import logging

from heatclient._i18n import _
from heatclient import exc

logger = logging.getLogger(__name__)


def clear_hook(hc, stack_id, resource_name, hook_type):
    try:
        hc.resources.signal(
            stack_id=stack_id,
            resource_name=resource_name,
            data={'unset_hook': hook_type})
    except exc.HTTPNotFound:
        logger.error(
            "Stack %(stack)s or resource %(resource)s "
            "not found for hook %(hook_type)s",
            {'resource': resource_name, 'stack': stack_id,
             'hook_type': hook_type})


def clear_wildcard_hooks(hc, stack_id, stack_patterns, hook_type,
                         resource_pattern):
    if stack_patterns:
        for resource in hc.resources.list(stack_id):
            res_name = resource.resource_name
            if fnmatch.fnmatchcase(res_name, stack_patterns[0]):
                nested_stack = hc.resources.get(
                    stack_id=stack_id,
                    resource_name=res_name)
                clear_wildcard_hooks(
                    hc,
                    nested_stack.physical_resource_id,
                    stack_patterns[1:], hook_type, resource_pattern)
    else:
        for resource in hc.resources.list(stack_id):
            res_name = resource.resource_name
            if fnmatch.fnmatchcase(res_name, resource_pattern):
                clear_hook(hc, stack_id, res_name, hook_type)


def get_hook_type_via_status(hc, stack_id):
    # Figure out if the hook should be pre-create, pre-update or
    # pre-delete based on the stack status, also sanity assertions
    # that we're in-progress.
    try:
        stack = hc.stacks.get(stack_id=stack_id)
    except exc.HTTPNotFound:
        raise exc.CommandError(_('Stack not found: %s') % stack_id)
    else:
        if 'IN_PROGRESS' not in stack.stack_status:
            raise exc.CommandError(_('Stack status %s not IN_PROGRESS') %
                                   stack.stack_status)

    if 'CREATE' in stack.stack_status:
        hook_type = 'pre-create'
    elif 'UPDATE' in stack.stack_status:
        hook_type = 'pre-update'
    elif 'DELETE' in stack.stack_status:
        hook_type = 'pre-delete'
    else:
        raise exc.CommandError(_('Unexpected stack status %s, '
                                 'only create, update and delete supported')
                               % stack.stack_status)
    return hook_type



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

from unittest import mock

from heatclient.osc.v1 import build_info as osc_build_info
from heatclient.osc.v1 import common
from heatclient.tests.unit.osc.v1 import fakes as orchestration_fakes


class TestBuildInfo(orchestration_fakes.TestOrchestrationv1):
    response = {"api": {
        "revision": "{api_build_revision}"
        },
        "engine": {
            "revision": "{engine_build_revision}"
        }
    }

    def setUp(self):
        super(TestBuildInfo, self).setUp()
        self.cmd = osc_build_info.BuildInfo(self.app, None)
        self.mock_client = self.app.client_manager.orchestration
        self.mock_client.build_info.build_info = mock.Mock(
            return_value=self.response)

    def test_build_info(self):
        arglist = []
        parsed_args = self.check_parser(self.cmd, arglist, [])
        columns, data = self.cmd.take_action(parsed_args)
        self.mock_client.build_info.build_info.assert_called_with()
        self.assertEqual(['api', 'engine'], columns)
        self.assertEqual([
            common.JsonColumn({"revision": "{api_build_revision}"}),
            common.JsonColumn({"revision": "{engine_build_revision}"})
        ], list(data))



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

"""Orchestration v1 software config action implementations"""

import logging

from osc_lib.command import command
from osc_lib import exceptions as exc
from osc_lib import utils
from urllib import request
import yaml

from heatclient._i18n import _
from heatclient.common import format_utils
from heatclient.common import template_format
from heatclient.common import utils as heat_utils
from heatclient import exc as heat_exc


class DeleteConfig(command.Command):
    """Delete software configs"""

    log = logging.getLogger(__name__ + ".DeleteConfig")

    def get_parser(self, prog_name):
        parser = super(DeleteConfig, self).get_parser(prog_name)
        parser.add_argument(
            'config',
            metavar='<config>',
            nargs='+',
            help=_('IDs of the software configs to delete')
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)

        heat_client = self.app.client_manager.orchestration
        return _delete_config(heat_client, parsed_args)


def _delete_config(heat_client, args):
    failure_count = 0

    for config_id in args.config:
        try:
            heat_client.software_configs.delete(
                config_id=config_id)
        except Exception as e:
            if isinstance(e, heat_exc.HTTPNotFound):
                print(_('Software config with ID %s not found') % config_id)
            failure_count += 1
            continue

    if failure_count:
        raise exc.CommandError(_('Unable to delete %(count)s of the '
                                 '%(total)s software configs.') %
                               {'count': failure_count,
                                'total': len(args.config)})


class ListConfig(command.Lister):
    """List software configs"""

    log = logging.getLogger(__name__ + ".ListConfig")

    def get_parser(self, prog_name):
        parser = super(ListConfig, self).get_parser(prog_name)
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            type=int,
            help=_('Limit the number of configs returned')
        )
        parser.add_argument(
            '--marker',
            metavar='<id>',
            help=_('Return configs that appear after the given config ID')
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        heat_client = self.app.client_manager.orchestration
        return _list_config(heat_client, parsed_args)


def _list_config(heat_client, args):
    kwargs = {}
    if args.limit:
        kwargs['limit'] = args.limit
    if args.marker:
        kwargs['marker'] = args.marker
    scs = heat_client.software_configs.list(**kwargs)

    columns = ['id', 'name', 'group', 'creation_time']
    return (columns, (utils.get_item_properties(s, columns) for s in scs))


class CreateConfig(format_utils.JsonFormat):
    """Create software config"""

    log = logging.getLogger(__name__ + ".CreateConfig")

    def get_parser(self, prog_name):
        parser = super(CreateConfig, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<config-name>',
            help=_('Name of the software config to create')
        )
        parser.add_argument(
            '--config-file',
            metavar='<config-file>',
            help=_('Path to JSON/YAML containing map defining '
                   '<inputs>, <outputs>, and <options>')
        )
        parser.add_argument(
            '--definition-file',
            metavar='<destination-file>',
            help=_('Path to software config script/data')
        )
        parser.add_argument(
            '--group',
            metavar='<group>',
            default='Heat::Ungrouped',
            help=_('Group name of tool expected by the software config')
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        heat_client = self.app.client_manager.orchestration
        return _create_config(heat_client, parsed_args)


def _create_config(heat_client, args):
    config = {
        'group': args.group,
        'config': ''
    }

    defn = {}
    if args.definition_file:
        defn_url = heat_utils.normalise_file_path_to_url(
            args.definition_file)
        defn_raw = request.urlopen(defn_url).read() or '{}'
        defn = yaml.load(defn_raw, Loader=template_format.yaml_loader)

    config['inputs'] = defn.get('inputs', [])
    config['outputs'] = defn.get('outputs', [])
    config['options'] = defn.get('options', {})

    if args.config_file:
        config_url = heat_utils.normalise_file_path_to_url(
            args.config_file)
        config['config'] = request.urlopen(config_url).read()

    # build a mini-template with a config resource and validate it
    validate_template = {
        'heat_template_version': '2013-05-23',
        'resources': {
            args.name: {
                'type': 'OS::Heat::SoftwareConfig',
                'properties': config
            }
        }
    }
    heat_client.stacks.validate(template=validate_template)

    config['name'] = args.name
    sc = heat_client.software_configs.create(**config).to_dict()
    rows = list(sc.values())
    columns = list(sc.keys())
    return columns, rows


class ShowConfig(format_utils.YamlFormat):
    """Show software config details"""

    log = logging.getLogger(__name__ + ".ShowConfig")

    def get_parser(self, prog_name):
        parser = super(ShowConfig, self).get_parser(prog_name)
        parser.add_argument(
            'config',
            metavar='<config>',
            help=_('ID of the config')
        )
        parser.add_argument(
            '--config-only',
            default=False,
            action="store_true",
            help=_('Only display the value of the <config> property.')
        )
        return parser

    def take_action(self, parsed_args):
        self.log.debug("take_action(%s)", parsed_args)
        heat_client = self.app.client_manager.orchestration
        return _show_config(heat_client, config_id=parsed_args.config,
                            config_only=parsed_args.config_only)


def _show_config(heat_client, config_id, config_only):
    try:
        sc = heat_client.software_configs.get(config_id=config_id)
    except heat_exc.HTTPNotFound:
        raise exc.CommandError(_('Configuration not found: %s') % config_id)

    columns = None
    rows = None

    if config_only:
        print(sc.config)
    else:
        columns = (
            'id',
            'name',
            'group',
            'config',
            'inputs',
            'outputs',
            'options',
            'creation_time',
        )
        rows = utils.get_dict_properties(sc.to_dict(), columns)

    return columns, rows



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import copy
from unittest import mock

from osc_lib import exceptions as exc

from heatclient import exc as heat_exc
from heatclient.osc.v1 import software_deployment
from heatclient.tests.unit.osc.v1 import fakes as orchestration_fakes
from heatclient.v1 import software_configs
from heatclient.v1 import software_deployments


class TestDeployment(orchestration_fakes.TestOrchestrationv1):
    def setUp(self):
        super(TestDeployment, self).setUp()
        self.mock_client = self.app.client_manager.orchestration
        self.config_client = self.mock_client.software_configs
        self.sd_client = self.mock_client.software_deployments


class TestDeploymentCreate(TestDeployment):

    server_id = '1234'
    config_id = '5678'
    deploy_id = '910'

    config = {
        'name': 'my_deploy',
        'group': 'strict',
        'config': '#!/bin/bash',
        'inputs': [],
        'outputs': [],
        'options': [],
        'id': config_id,
    }

    deployment = {
        'server_id': server_id,
        'input_values': {},
        'action': 'UPDATE',
        'status': 'IN_PROGRESS',
        'status_reason': None,
        'signal_id': 'signal_id',
        'config_id': config_id,
        'id': deploy_id,
    }

    config_defaults = {
        'group': 'Heat::Ungrouped',
        'config': '',
        'options': {},
        'inputs': [
            {
                'name': 'deploy_server_id',
                'description': 'ID of the server being deployed to',
                'type': 'String',
                'value': server_id,
            },
            {
                'name': 'deploy_action',
                'description': 'Name of the current action being deployed',
                'type': 'String',
                'value': 'UPDATE',
            },
            {
                'name': 'deploy_signal_transport',
                'description': 'How the server should signal to heat with the '
                               'deployment output values.',
                'type': 'String',
                'value': 'TEMP_URL_SIGNAL',
            },
            {
                'name': 'deploy_signal_id',
                'description': 'ID of signal to use for signaling output '
                               'values',
                'type': 'String',
                'value': 'signal_id',
            },
            {
                'name': 'deploy_signal_verb',
                'description': 'HTTP verb to use for signaling output values',
                'type': 'String',
                'value': 'PUT',
            },
        ],
        'outputs': [],
        'name': 'my_deploy',
    }

    deploy_defaults = {
        'config_id': config_id,
        'server_id': server_id,
        'action': 'UPDATE',
        'status': 'IN_PROGRESS',
    }

    def setUp(self):
        super(TestDeploymentCreate, self).setUp()
        self.cmd = software_deployment.CreateDeployment(self.app, None)
        self.config_client.create.return_value = \
            software_configs.SoftwareConfig(None, self.config)
        self.config_client.get.return_value = \
            software_configs.SoftwareConfig(None, self.config)
        self.sd_client.create.return_value = \
            software_deployments.SoftwareDeployment(None, self.deployment)

    @mock.patch('heatclient.common.deployment_utils.build_signal_id',
                return_value='signal_id')
    def test_deployment_create(self, mock_build):
        arglist = ['my_deploy', '--server', self.server_id]
        expected_cols = ('action', 'config_id', 'id',  'input_values',
                         'server_id', 'signal_id', 'status', 'status_reason')
        expected_data = ('UPDATE', self.config_id, self.deploy_id, {},
                         self.server_id, 'signal_id', 'IN_PROGRESS', None)
        parsed_args = self.check_parser(self.cmd, arglist, [])

        columns, data = self.cmd.take_action(parsed_args)

        self.config_client.create.assert_called_with(**self.config_defaults)
        self.sd_client.create.assert_called_with(
            **self.deploy_defaults)
        self.assertEqual(expected_cols, columns)
        self.assertEqual(expected_data, data)

    @mock.patch('heatclient.common.deployment_utils.build_signal_id',
                return_value='signal_id')
    def test_deployment_create_with_config(self, mock_build):
        arglist = ['my_deploy', '--server', self.server_id,
                   '--config', self.config_id]
        config = copy.deepcopy(self.config_defaults)
        config['config'] = '#!/bin/bash'
        config['group'] = 'strict'
        parsed_args = self.check_parser(self.cmd, arglist, [])

        columns, data = self.cmd.take_action(parsed_args)

        self.config_client.get.assert_called_with(self.config_id)
        self.config_client.create.assert_called_with(**config)
        self.sd_client.create.assert_called_with(
            **self.deploy_defaults)

    def test_deployment_create_config_not_found(self):
        arglist = ['my_deploy', '--server', self.server_id,
                   '--config', 'bad_id']
        self.config_client.get.side_effect = heat_exc.HTTPNotFound
        parsed_args = self.check_parser(self.cmd, arglist, [])

        self.assertRaises(exc.CommandError, self.cmd.take_action, parsed_args)

    def test_deployment_create_no_signal(self):
        arglist = ['my_deploy', '--server', self.server_id,
                   '--signal-transport', 'NO_SIGNAL']
        config = copy.deepcopy(self.config_defaults)
        config['inputs'] = config['inputs'][:-2]
        config['inputs'][2]['value'] = 'NO_SIGNAL'
        parsed_args = self.check_parser(self.cmd, arglist, [])

        columns, data = self.cmd.take_action(parsed_args)

        self.config_client.create.assert_called_with(**config)
        self.sd_client.create.assert_called_with(
            **self.deploy_defaults)

    @mock.patch('heatclient.common.deployment_utils.build_signal_id',
                return_value='signal_id')
    def test_deployment_create_invalid_signal_transport(self, mock_build):
        arglist = ['my_deploy', '--server', self.server_id,
                   '--signal-transport', 'A']
        parsed_args = self.check_parser(self.cmd, arglist, [])

        self.assertRaises(heat_exc.CommandError,
                          self.cmd.take_action, parsed_args)

    @mock.patch('heatclient.common.deployment_utils.build_signal_id',
                return_value='signal_id')
    def test_deployment_create_input_value(self, mock_build):
        arglist = ['my_deploy', '--server', self.server_id,
                   '--input-value', 'foo=bar']
        config = copy.deepcopy(self.config_defaults)
        config['inputs'].insert(
            0, {'name': 'foo', 'type': 'String', 'value': 'bar'})
        parsed_args = self.check_parser(self.cmd, arglist, [])

        columns, data = self.cmd.take_action(parsed_args)

        self.config_client.create.assert_called_with(**config)
        self.sd_client.create.assert_called_with(
            **self.deploy_defaults)

    @mock.patch('heatclient.common.deployment_utils.build_signal_id',
                return_value='signal_id')
    def test_deployment_create_action(self, mock_build):
        arglist = ['my_deploy', '--server', self.server_id,
                   '--action', 'DELETE']
        config = copy.deepcopy(self.config_defaults)
        config['inputs'][1]['value'] = 'DELETE'
        deploy = copy.deepcopy(self.deploy_defaults)
        deploy['action'] = 'DELETE'
        parsed_args = self.check_parser(self.cmd, arglist, [])

        columns, data = self.cmd.take_action(parsed_args)

        self.config_client.create.assert_called_with(**config)
        self.sd_client.create.assert_called_with(**deploy)


class TestDeploymentDelete(TestDeployment):

    def setUp(self):
        super(TestDeploymentDelete, self).setUp()
        self.cmd = software_deployment.DeleteDeployment(self.app, None)

    def test_deployment_delete_success(self):
        arglist = ['test_deployment']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.cmd.take_action(parsed_args)
        self.sd_client.delete.assert_called_with(
            deployment_id='test_deployment')

    def test_deployment_delete_multiple(self):
        arglist = ['test_deployment', 'test_deployment2']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.cmd.take_action(parsed_args)
        self.sd_client.delete.assert_has_calls(
            [mock.call(deployment_id='test_deployment'),
             mock.call(deployment_id='test_deployment2')])

    def test_deployment_delete_not_found(self):
        arglist = ['test_deployment', 'test_deployment2']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.sd_client.delete.side_effect = heat_exc.HTTPNotFound()
        error = self.assertRaises(
            exc.CommandError, self.cmd.take_action, parsed_args)
        self.assertIn("Unable to delete 2 of the 2 deployments.", str(error))

    def test_deployment_config_delete_failed(self):
        arglist = ['test_deployment']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.config_client.delete.side_effect = heat_exc.HTTPNotFound()
        self.assertIsNone(self.cmd.take_action(parsed_args))


class TestDeploymentList(TestDeployment):

    columns = ['id', 'config_id', 'server_id', 'action', 'status']

    data = {"software_deployments": [
        {
            "status": "COMPLETE",
            "server_id": "ec14c864-096e-4e27-bb8a-2c2b4dc6f3f5",
            "config_id": "8da95794-2ad9-4979-8ae5-739ce314c5cd",
            "output_values": {
                "deploy_stdout": "Writing to /tmp/barmy Written to /tmp/barmy",
                "deploy_stderr": "+ echo Writing to /tmp/barmy\n+ echo fu\n+ c"
                                 "at /tmp/barmy\n+ echo -n The file /tmp/barmy"
                                 "contains for server ec14c864-096e-4e27-bb8a-"
                                 "2c2b4dc6f3f5 during CREATE\n+"
                                 "echo Output to stderr\nOutput to stderr\n",
                "deploy_status_code": 0,
                "result": "The file /tmp/barmy contains fu for server "
                          "ec14c864-096e-4e27-bb8a-2c2b4dc6f3f5 during CREATE"
            },
            "input_values": None,
            "action": "CREATE",
            "status_reason": "Outputs received",
            "id": "ef422fa5-719a-419e-a10c-72e3a367b0b8",
            "creation_time": "2015-01-31T15:12:36Z",
            "updated_time": "2015-01-31T15:18:21Z"
        }
    ]
    }

    def setUp(self):
        super(TestDeploymentList, self).setUp()
        self.cmd = software_deployment.ListDeployment(self.app, None)
        self.sd_client.list = mock.MagicMock(return_value=[self.data])

    def test_deployment_list(self):
        arglist = []
        parsed_args = self.check_parser(self.cmd, arglist, [])
        columns, data = self.cmd.take_action(parsed_args)
        self.sd_client.list.assert_called_with()
        self.assertEqual(self.columns, columns)

    def test_deployment_list_server(self):
        kwargs = {}
        kwargs['server_id'] = 'ec14c864-096e-4e27-bb8a-2c2b4dc6f3f5'
        arglist = ['--server', 'ec14c864-096e-4e27-bb8a-2c2b4dc6f3f5']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        columns, data = self.cmd.take_action(parsed_args)
        self.sd_client.list.assert_called_with(**kwargs)
        self.assertEqual(self.columns, columns)

    def test_deployment_list_long(self):
        kwargs = {}
        cols = ['id', 'config_id', 'server_id', 'action', 'status',
                'creation_time', 'status_reason']
        arglist = ['--long']
        parsed_args = self.check_parser(self.cmd, arglist, [])

        columns, data = self.cmd.take_action(parsed_args)

        self.sd_client.list.assert_called_with(**kwargs)
        self.assertEqual(cols, columns)


class TestDeploymentShow(TestDeployment):
    get_response = {"software_deployment": {
        "status": "IN_PROGRESS",
        "server_id": "ec14c864-096e-4e27-bb8a-2c2b4dc6f3f5",
        "config_id": "3d5ec2a8-7004-43b6-a7f6-542bdbe9d434",
        "output_values": 'null',
        "input_values": 'null',
        "action": "CREATE",
        "status_reason": "Deploy data available",
        "id": "06e87bcc-33a2-4bce-aebd-533e698282d3",
        "creation_time": "2015-01-31T15:12:36Z",
        "updated_time": "2015-01-31T15:18:21Z"
    }}

    def setUp(self):
        super(TestDeploymentShow, self).setUp()
        self.cmd = software_deployment.ShowDeployment(self.app, None)

    def test_deployment_show(self):
        arglist = ['my_deployment']
        cols = ['id', 'server_id', 'config_id', 'creation_time',
                'updated_time', 'status', 'status_reason',
                'input_values', 'action']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.sd_client.get.return_value = \
            software_deployments.SoftwareDeployment(
                None, self.get_response)
        columns, data = self.cmd.take_action(parsed_args)
        self.sd_client.get.assert_called_with(**{
            'deployment_id': 'my_deployment',
        })
        self.assertEqual(cols, columns)

    def test_deployment_show_long(self):
        arglist = ['my_deployment', '--long']
        cols = ['id', 'server_id', 'config_id', 'creation_time',
                'updated_time', 'status', 'status_reason',
                'input_values', 'action', 'output_values']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.sd_client.get.return_value = \
            software_deployments.SoftwareDeployment(
                None, self.get_response)
        columns, data = self.cmd.take_action(parsed_args)
        self.sd_client.get.assert_called_once_with(**{
            'deployment_id': 'my_deployment',
        })
        self.assertEqual(cols, columns)

    def test_deployment_not_found(self):
        arglist = ['my_deployment']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.sd_client.get.side_effect = heat_exc.HTTPNotFound()
        self.assertRaises(
            exc.CommandError,
            self.cmd.take_action,
            parsed_args)


class TestDeploymentMetadataShow(TestDeployment):

    def setUp(self):
        super(TestDeploymentMetadataShow, self).setUp()
        self.cmd = software_deployment.ShowMetadataDeployment(self.app, None)
        self.sd_client.metadata.return_value = {}

    def test_deployment_show_metadata(self):
        arglist = ['ec14c864-096e-4e27-bb8a-2c2b4dc6f3f5']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.cmd.take_action(parsed_args)
        self.sd_client.metadata.assert_called_with(
            server_id='ec14c864-096e-4e27-bb8a-2c2b4dc6f3f5')


class TestDeploymentOutputShow(TestDeployment):

    get_response = {
        "status": "IN_PROGRESS",
        "server_id": "ec14c864-096e-4e27-bb8a-2c2b4dc6f3f5",
        "config_id": "3d5ec2a8-7004-43b6-a7f6-542bdbe9d434",
        "output_values": None,
        "input_values": None,
        "action": "CREATE",
        "status_reason": "Deploy data available",
        "id": "06e87bcc-33a2-4bce-aebd-533e698282d3",
        "creation_time": "2015-01-31T15:12:36Z",
        "updated_time": "2015-01-31T15:18:21Z"
    }

    def setUp(self):
        super(TestDeploymentOutputShow, self).setUp()
        self.cmd = software_deployment.ShowOutputDeployment(self.app, None)

    def test_deployment_output_show(self):
        arglist = ['85c3a507-351b-4b28-a7d8-531c8d53f4e6', '--all', '--long']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.sd_client.get.return_value = \
            software_deployments.SoftwareDeployment(
                None, self.get_response)
        self.cmd.take_action(parsed_args)
        self.sd_client.get.assert_called_with(**{
            'deployment_id': '85c3a507-351b-4b28-a7d8-531c8d53f4e6'
            })

    def test_deployment_output_show_invalid(self):
        arglist = ['85c3a507-351b-4b28-a7d8-531c8d53f4e6']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        error = self.assertRaises(
            exc.CommandError,
            self.cmd.take_action,
            parsed_args)
        self.assertIn('either <output-name> or --all argument is needed',
                      str(error))

    def test_deployment_output_show_not_found(self):
        arglist = ['85c3a507-351b-4b28-a7d8-531c8d53f4e6', '--all']
        parsed_args = self.check_parser(self.cmd, arglist, [])
        self.sd_client.get.side_effect = heat_exc.HTTPNotFound()
        self.assertRaises(
            exc.CommandError,
            self.cmd.take_action,
            parsed_args)
